#include "Calculator_objects.h"
#include "Calculator_resources.h"
#include "built_in.h"

// TFT module connections
char TFT_DataPort at LATD;
char TFT_DataPort_Direction at TRISD;
sbit TFT_RST at LATC1_bit;
sbit TFT_BLED at LATC2_bit;
sbit TFT_RS at LATB15_bit;
sbit TFT_CS at LATC3_bit;
sbit TFT_RD at LATD12_bit;
sbit TFT_WR at LATD13_bit;
sbit TFT_RST_Direction at TRISC1_bit;
sbit TFT_BLED_Direction at TRISC2_bit;
sbit TFT_RS_Direction at TRISB15_bit;
sbit TFT_CS_Direction at TRISC3_bit;
sbit TFT_RD_Direction at TRISD12_bit;
sbit TFT_WR_Direction at TRISD13_bit;
// End Glcd module connections

// Touch Panel module connections
sbit DriveX_Left at LATB13_bit;
sbit DriveX_Right at LATB11_bit;
sbit DriveY_Up at LATB12_bit;
sbit DriveY_Down at LATB10_bit;
sbit DriveX_Left_Direction at TRISB13_bit;
sbit DriveX_Right_Direction at TRISB11_bit;
sbit DriveY_Up_Direction at TRISB12_bit;
sbit DriveY_Down_Direction at TRISB10_bit;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
char PenDown;
void *PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

// set index
void Set_Index(unsigned short index) {
  TFT_RS = 0;
  Lo(LATD) = index;
  TFT_WR = 0;
  TFT_WR = 1;
}

// write command
void Write_Command(unsigned short cmd) {
  TFT_RS = 1;
  Lo(LATD) = cmd;
  TFT_WR = 0;
  TFT_WR = 1;
}

// write data
void Write_Data(unsigned int _data) {
  TFT_RS = 1;
  Lo(LATE) = Hi(_data);
  Lo(LATD) = Lo(_data);
  TFT_WR = 0;
  TFT_WR = 1;
}

static void InitializeTouchPanel() {
  AD1PCFGL = 0xCFFF;
  AD1PCFGH = 0xCFFF;
  TRISE = 0;
  TRISD = 0;
  CLKDIVbits.PLLPRE = 0;
  PLLFBD = 0x30;
  CLKDIVbits.PLLPOST = 1;
  Delay_ms(150);
  ADC1_Init();
  TFT_Set_Active(Set_Index,Write_Command,Write_Data);
  TFT_Init(320, 240);

  TP_TFT_Init(320, 240, 13, 12);                                  // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(900);                              // Set touch panel ADC threshold

  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}


/////////////////////////
  TScreen *CurrentScreen;

  TScreen                Screen1;
  TButton                Button1;
  char Button1_Caption[2] = "7";

  TButton                Button5;
  char Button5_Caption[2] = "4";

  TButton                Button6;
  char Button6_Caption[2] = "1";

  TButton                Button7;
  char Button7_Caption[2] = "0";

  TButton                Button2;
  char Button2_Caption[2] = "8";

  TButton                Button3;
  char Button3_Caption[2] = "5";

  TButton                Button4;
  char Button4_Caption[2] = "2";

  TButton                Button8;
  char Button8_Caption[4] = "+/-";

  TButton                Button9;
  char Button9_Caption[2] = "9";

  TButton                Button10;
  char Button10_Caption[2] = "6";

  TButton                Button11;
  char Button11_Caption[2] = "3";

  TButton                Button13;
  char Button13_Caption[2] = "/";

  TButton                Button14;
  char Button14_Caption[2] = "*";

  TButton                Button15;
  char Button15_Caption[2] = "-";

  TButton                Button16;
  char Button16_Caption[2] = "+";

  TBox                   Box1;
  TButton                Button17;
  char Button17_Caption[4] = "DEL";

  TButton                Button18;
  char Button18_Caption[2] = "C";

  TButton                Button20;
  char Button20_Caption[2] = "=";

  TLabel                 Label1;
  char Label1_Caption[20] = " ";

  TLabel                 Label2;
  char Label2_Caption[2] = " ";

  TLabel                 Label3;
  char Label3_Caption[20] = " ";

  TLabel                 Label4;
  char Label4_Caption[2] = "=";

  TBox                   Box2;
  TLabel                 Label5;
  char Label5_Caption[14] = "mE Calculator";

  TLabel                 Label6;
  char Label6_Caption[18] = " ";

  TButton                *Screen0_Buttons[18];
  TLabel                 *Screen0_Labels[6];
  TBox                   *Screen0_Boxes[2];

  TScreen                Screen2;
  TLabel                 Label10;
  char Label10_Caption[23] = "mikroE Calculator Demo";

  TLabel                 Label11;
  char Label11_Caption[50] = "This is a simple version of calculator. Feel free";

  TButton_Round          ButtonRound1;
  char ButtonRound1_Caption[26] = "Continue to mE Calculator";

  TLabel                 Label7;
  char Label7_Caption[48] = "to try it out. Note that input values should be";

  TLabel                 Label8;
  char Label8_Caption[44] = "within int/integer values (-32768 - 32767).";

  TButton_Round          *Screen1_Buttons_Round[1];
  TLabel                 *Screen1_Labels[4];




static void InitializeObjects() {
  Screen1.Color                     = 0x0000;
  Screen1.Width                     = 240;
  Screen1.Height                    = 320;
  Screen1.ButtonsCount              = 18;
  Screen1.Buttons                   = Screen0_Buttons;
  Screen1.Buttons_RoundCount        = 0;
  Screen1.LabelsCount               = 6;
  Screen1.Labels                    = Screen0_Labels;
  Screen1.BoxesCount                = 2;
  Screen1.Boxes                     = Screen0_Boxes;
  Screen1.ObjectsCount              = 26;
  Screen0_Buttons[0]                = &Button1;
  Screen0_Buttons[1]                = &Button5;
  Screen0_Buttons[2]                = &Button6;
  Screen0_Buttons[3]                = &Button7;
  Screen0_Buttons[4]                = &Button2;
  Screen0_Buttons[5]                = &Button3;
  Screen0_Buttons[6]                = &Button4;
  Screen0_Buttons[7]                = &Button8;
  Screen0_Buttons[8]                = &Button9;
  Screen0_Buttons[9]                = &Button10;
  Screen0_Buttons[10]                = &Button11;
  Screen0_Buttons[11]                = &Button13;
  Screen0_Buttons[12]                = &Button14;
  Screen0_Buttons[13]                = &Button15;
  Screen0_Buttons[14]                = &Button16;
  Screen0_Boxes[0]                  = &Box1;
  Screen0_Buttons[15]                = &Button17;
  Screen0_Buttons[16]                = &Button18;
  Screen0_Buttons[17]                = &Button20;
  Screen0_Labels[0]                 = &Label1;
  Screen0_Labels[1]                 = &Label2;
  Screen0_Labels[2]                 = &Label3;
  Screen0_Labels[3]                 = &Label4;
  Screen0_Boxes[1]                  = &Box2;
  Screen0_Labels[4]                 = &Label5;
  Screen0_Labels[5]                 = &Label6;

  Screen2.Color                     = 0x0000;
  Screen2.Width                     = 320;
  Screen2.Height                    = 240;
  Screen2.ButtonsCount              = 0;
  Screen2.Buttons_RoundCount        = 1;
  Screen2.Buttons_Round             = Screen1_Buttons_Round;
  Screen2.LabelsCount               = 4;
  Screen2.Labels                    = Screen1_Labels;
  Screen2.BoxesCount                = 0;
  Screen2.ObjectsCount              = 5;
  Screen1_Labels[0]                 = &Label10;
  Screen1_Labels[1]                 = &Label11;
  Screen1_Buttons_Round[0]          = &ButtonRound1;
  Screen1_Labels[2]                 = &Label7;
  Screen1_Labels[3]                 = &Label8;


  Button1.OwnerScreen     = &Screen1;
  Button1.Order           = 0;
  Button1.Left            = 0;
  Button1.Top             = 162;
  Button1.Width           = 60;
  Button1.Height          = 40;
  Button1.Pen_Width       = 1;
  Button1.Pen_Color       = 0x0000;
  Button1.Visible         = 1;
  Button1.Active          = 1;
  Button1.Transparent     = 1;
  Button1.Caption         = Button1_Caption;
  Button1.FontName        = Arial21x24;
  Button1.Font_Color      = 0xFFFF;
  Button1.Gradient        = 1;
  Button1.Gradient_Orientation    = 0;
  Button1.Gradient_Start_Color    = 0x9492;
  Button1.Gradient_End_Color      = 0x0000;
  Button1.Color           = 0xC618;
  Button1.PressColEnabled = 1;
  Button1.Press_Color     = 0xC618;
  Button1.OnUpPtr        = 0;
  Button1.OnDownPtr      = 0;
  Button1.OnClickPtr     = Button1Click;
  Button1.OnPressPtr     = 0;

  Button5.OwnerScreen     = &Screen1;
  Button5.Order           = 1;
  Button5.Left            = 0;
  Button5.Top             = 202;
  Button5.Width           = 60;
  Button5.Height          = 40;
  Button5.Pen_Width       = 1;
  Button5.Pen_Color       = 0x0000;
  Button5.Visible         = 1;
  Button5.Active          = 1;
  Button5.Transparent     = 1;
  Button5.Caption         = Button5_Caption;
  Button5.FontName        = Arial21x24;
  Button5.Font_Color      = 0xFFFF;
  Button5.Gradient        = 1;
  Button5.Gradient_Orientation    = 0;
  Button5.Gradient_Start_Color    = 0x9492;
  Button5.Gradient_End_Color      = 0x0000;
  Button5.Color           = 0xC618;
  Button5.PressColEnabled = 1;
  Button5.Press_Color     = 0xC618;
  Button5.OnUpPtr        = 0;
  Button5.OnDownPtr      = 0;
  Button5.OnClickPtr     = Button5Click;
  Button5.OnPressPtr     = 0;

  Button6.OwnerScreen     = &Screen1;
  Button6.Order           = 2;
  Button6.Left            = 0;
  Button6.Top             = 242;
  Button6.Width           = 60;
  Button6.Height          = 40;
  Button6.Pen_Width       = 1;
  Button6.Pen_Color       = 0x0000;
  Button6.Visible         = 1;
  Button6.Active          = 1;
  Button6.Transparent     = 1;
  Button6.Caption         = Button6_Caption;
  Button6.FontName        = Arial21x24;
  Button6.Font_Color      = 0xFFFF;
  Button6.Gradient        = 1;
  Button6.Gradient_Orientation    = 0;
  Button6.Gradient_Start_Color    = 0x9492;
  Button6.Gradient_End_Color      = 0x0000;
  Button6.Color           = 0xC618;
  Button6.PressColEnabled = 1;
  Button6.Press_Color     = 0xC618;
  Button6.OnUpPtr        = 0;
  Button6.OnDownPtr      = 0;
  Button6.OnClickPtr     = Button6Click;
  Button6.OnPressPtr     = 0;

  Button7.OwnerScreen     = &Screen1;
  Button7.Order           = 3;
  Button7.Left            = 0;
  Button7.Top             = 282;
  Button7.Width           = 120;
  Button7.Height          = 40;
  Button7.Pen_Width       = 1;
  Button7.Pen_Color       = 0x0000;
  Button7.Visible         = 1;
  Button7.Active          = 1;
  Button7.Transparent     = 1;
  Button7.Caption         = Button7_Caption;
  Button7.FontName        = Arial21x24;
  Button7.Font_Color      = 0xFFFF;
  Button7.Gradient        = 1;
  Button7.Gradient_Orientation    = 0;
  Button7.Gradient_Start_Color    = 0x9492;
  Button7.Gradient_End_Color      = 0x0000;
  Button7.Color           = 0xC618;
  Button7.PressColEnabled = 1;
  Button7.Press_Color     = 0xC618;
  Button7.OnUpPtr        = 0;
  Button7.OnDownPtr      = 0;
  Button7.OnClickPtr     = Button7Click;
  Button7.OnPressPtr     = 0;

  Button2.OwnerScreen     = &Screen1;
  Button2.Order           = 4;
  Button2.Left            = 60;
  Button2.Top             = 162;
  Button2.Width           = 60;
  Button2.Height          = 40;
  Button2.Pen_Width       = 1;
  Button2.Pen_Color       = 0x0000;
  Button2.Visible         = 1;
  Button2.Active          = 1;
  Button2.Transparent     = 1;
  Button2.Caption         = Button2_Caption;
  Button2.FontName        = Arial21x24;
  Button2.Font_Color      = 0xFFFF;
  Button2.Gradient        = 1;
  Button2.Gradient_Orientation    = 0;
  Button2.Gradient_Start_Color    = 0x9492;
  Button2.Gradient_End_Color      = 0x0000;
  Button2.Color           = 0xC618;
  Button2.PressColEnabled = 1;
  Button2.Press_Color     = 0xC618;
  Button2.OnUpPtr        = 0;
  Button2.OnDownPtr      = 0;
  Button2.OnClickPtr     = Button2Click;
  Button2.OnPressPtr     = 0;

  Button3.OwnerScreen     = &Screen1;
  Button3.Order           = 5;
  Button3.Left            = 60;
  Button3.Top             = 202;
  Button3.Width           = 60;
  Button3.Height          = 40;
  Button3.Pen_Width       = 1;
  Button3.Pen_Color       = 0x0000;
  Button3.Visible         = 1;
  Button3.Active          = 1;
  Button3.Transparent     = 1;
  Button3.Caption         = Button3_Caption;
  Button3.FontName        = Arial21x24;
  Button3.Font_Color      = 0xFFFF;
  Button3.Gradient        = 1;
  Button3.Gradient_Orientation    = 0;
  Button3.Gradient_Start_Color    = 0x9492;
  Button3.Gradient_End_Color      = 0x0000;
  Button3.Color           = 0xC618;
  Button3.PressColEnabled = 1;
  Button3.Press_Color     = 0xC618;
  Button3.OnUpPtr        = 0;
  Button3.OnDownPtr      = 0;
  Button3.OnClickPtr     = Button3Click;
  Button3.OnPressPtr     = 0;

  Button4.OwnerScreen     = &Screen1;
  Button4.Order           = 6;
  Button4.Left            = 60;
  Button4.Top             = 242;
  Button4.Width           = 60;
  Button4.Height          = 40;
  Button4.Pen_Width       = 1;
  Button4.Pen_Color       = 0x0000;
  Button4.Visible         = 1;
  Button4.Active          = 1;
  Button4.Transparent     = 1;
  Button4.Caption         = Button4_Caption;
  Button4.FontName        = Arial21x24;
  Button4.Font_Color      = 0xFFFF;
  Button4.Gradient        = 1;
  Button4.Gradient_Orientation    = 0;
  Button4.Gradient_Start_Color    = 0x9492;
  Button4.Gradient_End_Color      = 0x0000;
  Button4.Color           = 0xC618;
  Button4.PressColEnabled = 1;
  Button4.Press_Color     = 0xC618;
  Button4.OnUpPtr        = 0;
  Button4.OnDownPtr      = 0;
  Button4.OnClickPtr     = Button4Click;
  Button4.OnPressPtr     = 0;

  Button8.OwnerScreen     = &Screen1;
  Button8.Order           = 7;
  Button8.Left            = 120;
  Button8.Top             = 282;
  Button8.Width           = 60;
  Button8.Height          = 40;
  Button8.Pen_Width       = 1;
  Button8.Pen_Color       = 0x0000;
  Button8.Visible         = 1;
  Button8.Active          = 1;
  Button8.Transparent     = 1;
  Button8.Caption         = Button8_Caption;
  Button8.FontName        = Arial21x24;
  Button8.Font_Color      = 0xFFFF;
  Button8.Gradient        = 1;
  Button8.Gradient_Orientation    = 0;
  Button8.Gradient_Start_Color    = 0x9492;
  Button8.Gradient_End_Color      = 0x0000;
  Button8.Color           = 0xC618;
  Button8.PressColEnabled = 1;
  Button8.Press_Color     = 0xC618;
  Button8.OnUpPtr        = 0;
  Button8.OnDownPtr      = 0;
  Button8.OnClickPtr     = Button8Click;
  Button8.OnPressPtr     = 0;

  Button9.OwnerScreen     = &Screen1;
  Button9.Order           = 8;
  Button9.Left            = 120;
  Button9.Top             = 162;
  Button9.Width           = 60;
  Button9.Height          = 40;
  Button9.Pen_Width       = 1;
  Button9.Pen_Color       = 0x0000;
  Button9.Visible         = 1;
  Button9.Active          = 1;
  Button9.Transparent     = 1;
  Button9.Caption         = Button9_Caption;
  Button9.FontName        = Arial21x24;
  Button9.Font_Color      = 0xFFFF;
  Button9.Gradient        = 1;
  Button9.Gradient_Orientation    = 0;
  Button9.Gradient_Start_Color    = 0x9492;
  Button9.Gradient_End_Color      = 0x0000;
  Button9.Color           = 0xC618;
  Button9.PressColEnabled = 1;
  Button9.Press_Color     = 0xC618;
  Button9.OnUpPtr        = 0;
  Button9.OnDownPtr      = 0;
  Button9.OnClickPtr     = Button9Click;
  Button9.OnPressPtr     = 0;

  Button10.OwnerScreen     = &Screen1;
  Button10.Order           = 9;
  Button10.Left            = 120;
  Button10.Top             = 202;
  Button10.Width           = 60;
  Button10.Height          = 40;
  Button10.Pen_Width       = 1;
  Button10.Pen_Color       = 0x0000;
  Button10.Visible         = 1;
  Button10.Active          = 1;
  Button10.Transparent     = 1;
  Button10.Caption         = Button10_Caption;
  Button10.FontName        = Arial21x24;
  Button10.Font_Color      = 0xFFFF;
  Button10.Gradient        = 1;
  Button10.Gradient_Orientation    = 0;
  Button10.Gradient_Start_Color    = 0x9492;
  Button10.Gradient_End_Color      = 0x0000;
  Button10.Color           = 0xC618;
  Button10.PressColEnabled = 1;
  Button10.Press_Color     = 0xC618;
  Button10.OnUpPtr        = 0;
  Button10.OnDownPtr      = 0;
  Button10.OnClickPtr     = Button10Click;
  Button10.OnPressPtr     = 0;

  Button11.OwnerScreen     = &Screen1;
  Button11.Order           = 10;
  Button11.Left            = 120;
  Button11.Top             = 242;
  Button11.Width           = 60;
  Button11.Height          = 40;
  Button11.Pen_Width       = 1;
  Button11.Pen_Color       = 0x0000;
  Button11.Visible         = 1;
  Button11.Active          = 1;
  Button11.Transparent     = 1;
  Button11.Caption         = Button11_Caption;
  Button11.FontName        = Arial21x24;
  Button11.Font_Color      = 0xFFFF;
  Button11.Gradient        = 1;
  Button11.Gradient_Orientation    = 0;
  Button11.Gradient_Start_Color    = 0x9492;
  Button11.Gradient_End_Color      = 0x0000;
  Button11.Color           = 0xC618;
  Button11.PressColEnabled = 1;
  Button11.Press_Color     = 0xC618;
  Button11.OnUpPtr        = 0;
  Button11.OnDownPtr      = 0;
  Button11.OnClickPtr     = Button11Click;
  Button11.OnPressPtr     = 0;

  Button13.OwnerScreen     = &Screen1;
  Button13.Order           = 11;
  Button13.Left            = 120;
  Button13.Top             = 123;
  Button13.Width           = 60;
  Button13.Height          = 40;
  Button13.Pen_Width       = 1;
  Button13.Pen_Color       = 0x0000;
  Button13.Visible         = 1;
  Button13.Active          = 1;
  Button13.Transparent     = 1;
  Button13.Caption         = Button13_Caption;
  Button13.FontName        = Arial21x24;
  Button13.Font_Color      = 0xFFFF;
  Button13.Gradient        = 1;
  Button13.Gradient_Orientation    = 0;
  Button13.Gradient_Start_Color    = 0x9492;
  Button13.Gradient_End_Color      = 0x0000;
  Button13.Color           = 0xC618;
  Button13.PressColEnabled = 1;
  Button13.Press_Color     = 0xC618;
  Button13.OnUpPtr        = 0;
  Button13.OnDownPtr      = 0;
  Button13.OnClickPtr     = Button13Click;
  Button13.OnPressPtr     = 0;

  Button14.OwnerScreen     = &Screen1;
  Button14.Order           = 12;
  Button14.Left            = 180;
  Button14.Top             = 123;
  Button14.Width           = 60;
  Button14.Height          = 40;
  Button14.Pen_Width       = 1;
  Button14.Pen_Color       = 0x0000;
  Button14.Visible         = 1;
  Button14.Active          = 1;
  Button14.Transparent     = 1;
  Button14.Caption         = Button14_Caption;
  Button14.FontName        = Arial21x24;
  Button14.Font_Color      = 0xFFFF;
  Button14.Gradient        = 1;
  Button14.Gradient_Orientation    = 0;
  Button14.Gradient_Start_Color    = 0x9492;
  Button14.Gradient_End_Color      = 0x0000;
  Button14.Color           = 0xC618;
  Button14.PressColEnabled = 1;
  Button14.Press_Color     = 0xC618;
  Button14.OnUpPtr        = 0;
  Button14.OnDownPtr      = 0;
  Button14.OnClickPtr     = Button14Click;
  Button14.OnPressPtr     = 0;

  Button15.OwnerScreen     = &Screen1;
  Button15.Order           = 13;
  Button15.Left            = 180;
  Button15.Top             = 162;
  Button15.Width           = 60;
  Button15.Height          = 40;
  Button15.Pen_Width       = 1;
  Button15.Pen_Color       = 0x0000;
  Button15.Visible         = 1;
  Button15.Active          = 1;
  Button15.Transparent     = 1;
  Button15.Caption         = Button15_Caption;
  Button15.FontName        = Arial21x24;
  Button15.Font_Color      = 0xFFFF;
  Button15.Gradient        = 1;
  Button15.Gradient_Orientation    = 0;
  Button15.Gradient_Start_Color    = 0x9492;
  Button15.Gradient_End_Color      = 0x0000;
  Button15.Color           = 0xC618;
  Button15.PressColEnabled = 1;
  Button15.Press_Color     = 0xC618;
  Button15.OnUpPtr        = 0;
  Button15.OnDownPtr      = 0;
  Button15.OnClickPtr     = Button15Click;
  Button15.OnPressPtr     = 0;

  Button16.OwnerScreen     = &Screen1;
  Button16.Order           = 14;
  Button16.Left            = 180;
  Button16.Top             = 202;
  Button16.Width           = 60;
  Button16.Height          = 40;
  Button16.Pen_Width       = 1;
  Button16.Pen_Color       = 0x0000;
  Button16.Visible         = 1;
  Button16.Active          = 1;
  Button16.Transparent     = 1;
  Button16.Caption         = Button16_Caption;
  Button16.FontName        = Arial21x24;
  Button16.Font_Color      = 0xFFFF;
  Button16.Gradient        = 1;
  Button16.Gradient_Orientation    = 0;
  Button16.Gradient_Start_Color    = 0x9492;
  Button16.Gradient_End_Color      = 0x0000;
  Button16.Color           = 0xC618;
  Button16.PressColEnabled = 1;
  Button16.Press_Color     = 0xC618;
  Button16.OnUpPtr        = 0;
  Button16.OnDownPtr      = 0;
  Button16.OnClickPtr     = Button16Click;
  Button16.OnPressPtr     = 0;

  Box1.OwnerScreen     = &Screen1;
  Box1.Order           = 15;
  Box1.Left            = 0;
  Box1.Top             = 26;
  Box1.Width           = 240;
  Box1.Height          = 97;
  Box1.Pen_Width       = 0;
  Box1.Pen_Color       = 0x0000;
  Box1.Visible         = 1;
  Box1.Active          = 1;
  Box1.Transparent     = 1;
  Box1.Gradient        = 0;
  Box1.Gradient_Orientation    = 0;
  Box1.Gradient_Start_Color    = 0x07E0;
  Box1.Gradient_End_Color      = 0x0540;
  Box1.Color           = 0x07E8;
  Box1.PressColEnabled = 0;
  Box1.Press_Color     = 0x8410;
  Box1.OnUpPtr        = 0;
  Box1.OnDownPtr      = 0;
  Box1.OnClickPtr     = 0;
  Box1.OnPressPtr     = 0;

  Button17.OwnerScreen     = &Screen1;
  Button17.Order           = 16;
  Button17.Left            = 0;
  Button17.Top             = 123;
  Button17.Width           = 60;
  Button17.Height          = 40;
  Button17.Pen_Width       = 1;
  Button17.Pen_Color       = 0x0000;
  Button17.Visible         = 1;
  Button17.Active          = 1;
  Button17.Transparent     = 1;
  Button17.Caption         = Button17_Caption;
  Button17.FontName        = Arial21x24;
  Button17.Font_Color      = 0xFFFF;
  Button17.Gradient        = 1;
  Button17.Gradient_Orientation    = 0;
  Button17.Gradient_Start_Color    = 0x9492;
  Button17.Gradient_End_Color      = 0x0000;
  Button17.Color           = 0xC618;
  Button17.PressColEnabled = 1;
  Button17.Press_Color     = 0xC618;
  Button17.OnUpPtr        = 0;
  Button17.OnDownPtr      = 0;
  Button17.OnClickPtr     = Button17Click;
  Button17.OnPressPtr     = 0;

  Button18.OwnerScreen     = &Screen1;
  Button18.Order           = 17;
  Button18.Left            = 60;
  Button18.Top             = 123;
  Button18.Width           = 60;
  Button18.Height          = 40;
  Button18.Pen_Width       = 1;
  Button18.Pen_Color       = 0x0000;
  Button18.Visible         = 1;
  Button18.Active          = 1;
  Button18.Transparent     = 1;
  Button18.Caption         = Button18_Caption;
  Button18.FontName        = Arial21x24;
  Button18.Font_Color      = 0xFFFF;
  Button18.Gradient        = 1;
  Button18.Gradient_Orientation    = 0;
  Button18.Gradient_Start_Color    = 0x9492;
  Button18.Gradient_End_Color      = 0x0000;
  Button18.Color           = 0xC618;
  Button18.PressColEnabled = 1;
  Button18.Press_Color     = 0xC618;
  Button18.OnUpPtr        = 0;
  Button18.OnDownPtr      = 0;
  Button18.OnClickPtr     = Button18Click;
  Button18.OnPressPtr     = 0;

  Button20.OwnerScreen     = &Screen1;
  Button20.Order           = 18;
  Button20.Left            = 180;
  Button20.Top             = 242;
  Button20.Width           = 60;
  Button20.Height          = 80;
  Button20.Pen_Width       = 1;
  Button20.Pen_Color       = 0x0000;
  Button20.Visible         = 1;
  Button20.Active          = 1;
  Button20.Transparent     = 1;
  Button20.Caption         = Button20_Caption;
  Button20.FontName        = Arial21x24;
  Button20.Font_Color      = 0xFFFF;
  Button20.Gradient        = 1;
  Button20.Gradient_Orientation    = 0;
  Button20.Gradient_Start_Color    = 0x9492;
  Button20.Gradient_End_Color      = 0x0000;
  Button20.Color           = 0xC618;
  Button20.PressColEnabled = 1;
  Button20.Press_Color     = 0xC618;
  Button20.OnUpPtr        = 0;
  Button20.OnDownPtr      = 0;
  Button20.OnClickPtr     = Button20Click;
  Button20.OnPressPtr     = 0;

  Label1.OwnerScreen    = &Screen1;
  Label1.Order          = 19;
  Label1.Left           = 4;
  Label1.Top            = 34;
  Label1.Width          = 21;
  Label1.Height         = 22;
  Label1.Visible        = 1;
  Label1.Active         = 1;
  Label1.Caption        = Label1_Caption;
  Label1.FontName       = Courier_New12x23;
  Label1.Font_Pos_Ver   = 32;
  Label1.Font_Color     = 0x0000;
  Label1.OnUpPtr        = 0;
  Label1.OnDownPtr      = 0;
  Label1.OnClickPtr     = 0;
  Label1.OnPressPtr     = 0;

  Label2.OwnerScreen    = &Screen1;
  Label2.Order          = 20;
  Label2.Left           = 4;
  Label2.Top            = 55;
  Label2.Width          = 21;
  Label2.Height         = 23;
  Label2.Visible        = 1;
  Label2.Active         = 1;
  Label2.Caption        = Label2_Caption;
  Label2.FontName       = Courier_New12x23;
  Label2.Font_Pos_Ver   = 53;
  Label2.Font_Color     = 0x0000;
  Label2.OnUpPtr        = 0;
  Label2.OnDownPtr      = 0;
  Label2.OnClickPtr     = 0;
  Label2.OnPressPtr     = 0;

  Label3.OwnerScreen    = &Screen1;
  Label3.Order          = 21;
  Label3.Left           = 4;
  Label3.Top            = 75;
  Label3.Width          = 21;
  Label3.Height         = 22;
  Label3.Visible        = 1;
  Label3.Active         = 1;
  Label3.Caption        = Label3_Caption;
  Label3.FontName       = Courier_New12x23;
  Label3.Font_Pos_Ver   = 73;
  Label3.Font_Color     = 0x0000;
  Label3.OnUpPtr        = 0;
  Label3.OnDownPtr      = 0;
  Label3.OnClickPtr     = 0;
  Label3.OnPressPtr     = 0;

  Label4.OwnerScreen    = &Screen1;
  Label4.Order          = 22;
  Label4.Left           = 4;
  Label4.Top            = 96;
  Label4.Width          = 21;
  Label4.Height         = 22;
  Label4.Visible        = 1;
  Label4.Active         = 1;
  Label4.Caption        = Label4_Caption;
  Label4.FontName       = Courier_New12x23;
  Label4.Font_Pos_Ver   = 94;
  Label4.Font_Color     = 0x0000;
  Label4.OnUpPtr        = 0;
  Label4.OnDownPtr      = 0;
  Label4.OnClickPtr     = 0;
  Label4.OnPressPtr     = 0;

  Box2.OwnerScreen     = &Screen1;
  Box2.Order           = 23;
  Box2.Left            = 0;
  Box2.Top             = 0;
  Box2.Width           = 240;
  Box2.Height          = 26;
  Box2.Pen_Width       = 1;
  Box2.Pen_Color       = 0x0000;
  Box2.Visible         = 1;
  Box2.Active          = 1;
  Box2.Transparent     = 1;
  Box2.Gradient        = 1;
  Box2.Gradient_Orientation    = 0;
  Box2.Gradient_Start_Color    = 0x9492;
  Box2.Gradient_End_Color      = 0x0000;
  Box2.Color           = 0xC618;
  Box2.PressColEnabled = 0;
  Box2.Press_Color     = 0x8410;
  Box2.OnUpPtr        = 0;
  Box2.OnDownPtr      = 0;
  Box2.OnClickPtr     = 0;
  Box2.OnPressPtr     = 0;

  Label5.OwnerScreen    = &Screen1;
  Label5.Order          = 24;
  Label5.Left           = 56;
  Label5.Top            = 5;
  Label5.Width          = 123;
  Label5.Height         = 18;
  Label5.Visible        = 1;
  Label5.Active         = 1;
  Label5.Caption        = Label5_Caption;
  Label5.FontName       = Arial16x19;
  Label5.Font_Pos_Ver   = 3;
  Label5.Font_Color     = 0xFFFF;
  Label5.OnUpPtr        = 0;
  Label5.OnDownPtr      = 0;
  Label5.OnClickPtr     = 0;
  Label5.OnPressPtr     = 0;

  Label6.OwnerScreen    = &Screen1;
  Label6.Order          = 25;
  Label6.Left           = 28;
  Label6.Top            = 96;
  Label6.Width          = 21;
  Label6.Height         = 22;
  Label6.Visible        = 1;
  Label6.Active         = 1;
  Label6.Caption        = Label6_Caption;
  Label6.FontName       = Courier_New12x23;
  Label6.Font_Pos_Ver   = 94;
  Label6.Font_Color     = 0x0000;
  Label6.OnUpPtr        = 0;
  Label6.OnDownPtr      = 0;
  Label6.OnClickPtr     = 0;
  Label6.OnPressPtr     = 0;

  Label10.OwnerScreen    = &Screen2;
  Label10.Order          = 0;
  Label10.Left           = 22;
  Label10.Top            = 25;
  Label10.Width          = 276;
  Label10.Height         = 24;
  Label10.Visible        = 1;
  Label10.Active         = 1;
  Label10.Caption        = Label10_Caption;
  Label10.FontName       = Arial21x24;
  Label10.Font_Pos_Ver   = 23;
  Label10.Font_Color     = 0xF800;
  Label10.OnUpPtr        = 0;
  Label10.OnDownPtr      = 0;
  Label10.OnClickPtr     = 0;
  Label10.OnPressPtr     = 0;

  Label11.OwnerScreen    = &Screen2;
  Label11.Order          = 1;
  Label11.Left           = 18;
  Label11.Top            = 72;
  Label11.Width          = 289;
  Label11.Height         = 15;
  Label11.Visible        = 1;
  Label11.Active         = 1;
  Label11.Caption        = Label11_Caption;
  Label11.FontName       = Tahoma13x16;
  Label11.Font_Pos_Ver   = 70;
  Label11.Font_Color     = 0x07FF;
  Label11.OnUpPtr        = 0;
  Label11.OnDownPtr      = 0;
  Label11.OnClickPtr     = 0;
  Label11.OnPressPtr     = 0;

  ButtonRound1.OwnerScreen     = &Screen2;
  ButtonRound1.Order           = 2;
  ButtonRound1.Left            = 142;
  ButtonRound1.Top             = 178;
  ButtonRound1.Width           = 150;
  ButtonRound1.Height          = 38;
  ButtonRound1.Pen_Width       = 1;
  ButtonRound1.Pen_Color       = 0x0000;
  ButtonRound1.Visible         = 1;
  ButtonRound1.Active          = 1;
  ButtonRound1.Transparent     = 1;
  ButtonRound1.Caption         = ButtonRound1_Caption;
  ButtonRound1.FontName        = Tahoma13x13;
  ButtonRound1.Font_Color      = 0x0000;
  ButtonRound1.Gradient        = 1;
  ButtonRound1.Gradient_Orientation    = 0;
  ButtonRound1.Gradient_Start_Color    = 0xFFFF;
  ButtonRound1.Gradient_End_Color      = 0x8410;
  ButtonRound1.Color           = 0x8410;
  ButtonRound1.PressColEnabled = 1;
  ButtonRound1.Press_Color     = 0x8410;
  ButtonRound1.OnUpPtr        = 0;
  ButtonRound1.OnDownPtr      = 0;
  ButtonRound1.OnClickPtr     = ButtonRound1Click;
  ButtonRound1.OnPressPtr     = 0;

  Label7.OwnerScreen    = &Screen2;
  Label7.Order          = 3;
  Label7.Left           = 18;
  Label7.Top            = 92;
  Label7.Width          = 284;
  Label7.Height         = 15;
  Label7.Visible        = 1;
  Label7.Active         = 1;
  Label7.Caption        = Label7_Caption;
  Label7.FontName       = Tahoma13x16;
  Label7.Font_Pos_Ver   = 90;
  Label7.Font_Color     = 0x07FF;
  Label7.OnUpPtr        = 0;
  Label7.OnDownPtr      = 0;
  Label7.OnClickPtr     = 0;
  Label7.OnPressPtr     = 0;

  Label8.OwnerScreen    = &Screen2;
  Label8.Order          = 4;
  Label8.Left           = 20;
  Label8.Top            = 114;
  Label8.Width          = 270;
  Label8.Height         = 15;
  Label8.Visible        = 1;
  Label8.Active         = 1;
  Label8.Caption        = Label8_Caption;
  Label8.FontName       = Tahoma13x16;
  Label8.Font_Pos_Ver   = 112;
  Label8.Font_Color     = 0x07FF;
  Label8.OnUpPtr        = 0;
  Label8.OnDownPtr      = 0;
  Label8.OnClickPtr     = 0;
  Label8.OnPressPtr     = 0;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetButton(index)              CurrentScreen->Buttons[index]
#define GetRoundButton(index)         CurrentScreen->Buttons_Round[index]
#define GetLabel(index)               CurrentScreen->Labels[index]
#define GetBox(index)                 CurrentScreen->Boxes[index]


void DrawButton(TButton *Abutton) {
  if (Abutton->Visible == 1) {
    TFT_Set_Brush(Abutton->Transparent, Abutton->Color, Abutton->Gradient, Abutton->Gradient_Orientation, Abutton->Gradient_Start_Color, Abutton->Gradient_End_Color);
    TFT_Set_Pen(Abutton->Pen_Color, Abutton->Pen_Width);
    TFT_Rectangle(Abutton->Left, Abutton->Top, Abutton->Left + Abutton->Width - 1, Abutton->Top + Abutton->Height - 1);
    TFT_Set_Font(Abutton->FontName, Abutton->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text_Return_Pos(Abutton->Caption, Abutton->Left, Abutton->Top);
    TFT_Write_Text(Abutton->Caption, (Abutton->Left + ((Abutton->Width - caption_length) / 2)), (Abutton->Top + ((Abutton->Height - caption_height) / 2)));
  }
}

static void DrawButtons() {
  int i;
  TButton *local_button;

  for ( i = 0 ; i < CurrentScreen->ButtonsCount ; i++ ) {
    local_button = GetButton(i);
    DrawButton(local_button);
  }
}


void DrawRoundButton(TButton_Round *Around_button) {
    if (Around_button->Visible == 1) {
      TFT_Set_Brush(Around_button->Transparent, Around_button->Color, Around_button->Gradient, Around_button->Gradient_Orientation,
                    Around_button->Gradient_Start_Color, Around_button->Gradient_End_Color);
      TFT_Set_Pen(Around_button->Pen_Color, Around_button->Pen_Width);
      if (Around_button->Height > Around_button->Width) {
        TFT_Rectangle_Round_Edges(Around_button->Left + 1, Around_button->Top + 1,
          Around_button->Left + Around_button->Width - 2,
          Around_button->Top + Around_button->Height - 2, (Around_button->Width/4));
      }
      else
        if (Around_button->Width > Around_button->Height) {
          TFT_Rectangle_Round_Edges(Around_button->Left + 1, Around_button->Top + 1,
            Around_button->Left + Around_button->Width - 2,
            Around_button->Top + Around_button->Height - 2, (Around_button->Height/4));
        }
      TFT_Set_Font(Around_button->FontName, Around_button->Font_Color, FO_HORIZONTAL);
      TFT_Write_Text_Return_Pos(Around_button->Caption, Around_button->Left, Around_button->Top);
      TFT_Write_Text(Around_button->Caption, (Around_button->Left + ((Around_button->Width - caption_length) / 2)),
                    (Around_button->Top + ((Around_button->Height - caption_height) / 2)));
    }
}

static void DrawRoundButtons() {
  int i;
  TButton_Round *local_round_button;

  for ( i = 0 ; i < CurrentScreen->Buttons_RoundCount ; i++ ) {
    local_round_button = GetRoundButton(i);
    DrawRoundButton(local_round_button);
  }
}


void DrawLabel(TLabel *Alabel) {
  if (Alabel->Visible == 1) {
    TFT_Set_Font(Alabel->FontName, Alabel->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text(Alabel->Caption, Alabel->Left, Alabel->Top);
  }
}

static void DrawLabels() {
  int i;
  TLabel *local_label;

  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    DrawLabel(local_label);
  }
}


void DrawBox(TBox *ABox) {
  if (ABox->Visible == 1) {
    TFT_Set_Brush(ABox->Transparent, ABox->Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_Start_Color, ABox->Gradient_End_Color);
    TFT_Set_Pen(ABox->Pen_Color, ABox->Pen_Width);
    TFT_Rectangle(ABox->Left, ABox->Top, ABox->Left + ABox->Width - 1, ABox->Top + ABox->Height - 1);
  }
}

static void DrawBoxes() {
  int i;
  TBox *local_box;

  for ( i = 0 ; i < CurrentScreen->BoxesCount ; i++ ) {
    local_box = GetBox(i);
    DrawBox(local_box);
  }
}

void DrawScreen(TScreen *aScreen) {
  unsigned short order;
  unsigned short button_idx;
  TButton *local_button;
  unsigned short round_button_idx;
  TButton_Round *local_round_button;
  unsigned short label_idx;
  TLabel *local_label;
  unsigned short box_idx;
  TBox *local_box;
  order = 0;
  button_idx = 0;
  round_button_idx = 0;
  label_idx = 0;
  box_idx = 0;

  if ((display_width != aScreen->Width) || (display_height != aScreen->Height)) {
    TFT_BLED_Direction = 0;
    TFT_BLED           = 0;
    TFT_Init(aScreen->Width, aScreen->Height);
    TP_TFT_Init(aScreen->Width, aScreen->Height, 13, 12);                                  // Initialize touch panel
    TP_TFT_Set_ADC_Threshold(900);                              // Set touch panel ADC threshold
    TFT_Fill_Screen(aScreen->Color);
    display_width = aScreen->Width;
    display_height = aScreen->Height;
    TFT_BLED           = 1;
  }
  else
    TFT_Fill_Screen(aScreen->Color);

  CurrentScreen = aScreen;


  while (order < CurrentScreen->ObjectsCount) {
    if (button_idx < CurrentScreen->ButtonsCount) {
      local_button = GetButton(button_idx);
      if (order == local_button->Order) {
        button_idx++;
        order++;
        DrawButton(local_button);
      }
    }

    if (round_button_idx < CurrentScreen->Buttons_RoundCount) {
      local_round_button = GetRoundButton(round_button_idx);
      if (order == local_round_button->Order) {
        order++;
        round_button_idx++;
        DrawRoundButton(local_round_button);
      }
    }

    if (label_idx < CurrentScreen->LabelsCount) {
      local_label = GetLabel(label_idx);
      if (order == local_label->Order) {
        label_idx++;
        order++;
        DrawLabel(local_label);
      }
    }

    if (box_idx < CurrentScreen->BoxesCount) {
      local_box = GetBox(box_idx);
      if (order == local_box->Order) {
        box_idx++;
        order++;
        DrawBox(local_box);
      }
    }

  }
}

static void Process_TP_Press(unsigned int X, unsigned int Y) {
  int i;
  TButton *local_button;
  TButton *exec_button;
  short button_order;
  TButton_Round *local_round_button;
  TButton_Round *exec_round_button;
  short round_button_order;
  TLabel *local_label;
  TLabel *exec_label;
  short label_order;
  TBox *local_box;
  TBox *exec_box;
  short box_order;

  button_order        = -1;
  round_button_order  = -1;
  label_order         = -1;
  box_order           = -1;

  // Buttons
  for ( i = 0 ; i < CurrentScreen->ButtonsCount ; i++ ) {
    local_button = GetButton(i);
    if (local_button->Active == 1) {
      if (IsInsideObject(X, Y, local_button->Left, local_button->Top,
                         local_button->Width, local_button->Height) == 1) {
        button_order = local_button->Order;
        exec_button = local_button;
      }
    }
  }

  // Buttons with Round Edges
  for ( i = 0 ; i < CurrentScreen->Buttons_RoundCount ; i++ ) {
    local_round_button = GetRoundButton(i);
    if (local_round_button->Active == 1) {
      if (IsInsideObject(X, Y, local_round_button->Left, local_round_button->Top,
                         local_round_button->Width, local_round_button->Height) == 1) {
        round_button_order = local_round_button->Order;
        exec_round_button = local_round_button;
      }
    }
  }

  // Labels
  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  // Boxes
  for ( i = 0 ; i < CurrentScreen->BoxesCount ; i++ ) {
    local_box = GetBox(i);
    if (local_box->Active == 1) {
      if (IsInsideObject(X, Y, local_box->Left, local_box->Top,
                         local_box->Width, local_box->Height) == 1) {
        box_order = local_box->Order;
        exec_box = local_box;
      }
    }
  }

  i = -1;
  if (button_order > i)
    i = button_order;
  if (round_button_order > i)
    i = round_button_order;
  if (label_order > i)
    i = label_order;
  if (box_order > i)
    i = box_order;

  if (i != -1) {
    if (i == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->OnPressPtr != 0) {
          exec_button->OnPressPtr();
          return;
        }
      }
    }

    if (i == round_button_order) {
      if (exec_round_button->Active == 1) {
        if (exec_round_button->OnPressPtr != 0) {
          exec_round_button->OnPressPtr();
          return;
        }
      }
    }

    if (i == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnPressPtr != 0) {
          exec_label->OnPressPtr();
          return;
        }
      }
    }

    if (i == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->OnPressPtr != 0) {
          exec_box->OnPressPtr();
          return;
        }
      }
    }

  }
}

static void Process_TP_Up(unsigned int X, unsigned int Y) {
  int i;
  TButton *local_button;
  TButton *exec_button;
  short button_order;
  TButton_Round *local_round_button;
  TButton_Round *exec_round_button;
  short round_button_order;
  TLabel *local_label;
  TLabel *exec_label;
  short label_order;
  TBox *local_box;
  TBox *exec_box;
  short box_order;

  switch (PressedObjectType) {
    // Button
    case 0: {
      if (PressedObject != 0) {
        exec_button = (TButton*)PressedObject;
        if (exec_button->PressColEnabled == 1) {
          TFT_Set_Brush(exec_button->Transparent, exec_button->Color, exec_button->Gradient, exec_button->Gradient_Orientation, exec_button->Gradient_Start_Color, exec_button->Gradient_End_Color);
          TFT_Set_Pen(exec_button->Pen_Color, exec_button->Pen_Width);
          TFT_Rectangle(exec_button->Left, exec_button->Top, exec_button->Left + exec_button->Width - 1, exec_button->Top + exec_button->Height - 1);
          TFT_Set_Font(exec_button->FontName, exec_button->Font_Color, FO_HORIZONTAL);
          TFT_Write_Text_Return_Pos(exec_button->Caption, exec_button->Left, exec_button->Top);
          TFT_Write_Text(exec_button->Caption, (exec_button->Left + ((exec_button->Width - caption_length) / 2)), (exec_button->Top + ((exec_button->Height - caption_height) / 2)));
        }
        break;
      }
      break;
    }
    // Round Button
    case 1: {
      if (PressedObject != 0) {
        exec_round_button = (TButton_Round*)PressedObject;
        if (exec_round_button->PressColEnabled == 1) {
          TFT_Set_Brush(exec_round_button->Transparent, exec_round_button->Color, exec_round_button->Gradient, exec_round_button->Gradient_Orientation,
                        exec_round_button->Gradient_Start_Color, exec_round_button->Gradient_End_Color);
          TFT_Set_Pen(exec_round_button->Pen_Color, exec_round_button->Pen_Width);
          if (exec_round_button->Height > exec_round_button->Width)
            TFT_Rectangle_Round_Edges(exec_round_button->Left + 1, exec_round_button->Top + 1,
                                      exec_round_button->Left + exec_round_button->Width - 2,
                                      exec_round_button->Top + exec_round_button->Height - 2, (exec_round_button->Width / 4));
          else if (exec_round_button->Width > exec_round_button->Height)
            TFT_Rectangle_Round_Edges(exec_round_button->Left + 1, exec_round_button->Top + 1,
                                      exec_round_button->Left + exec_round_button->Width - 2,
                                      exec_round_button->Top + exec_round_button->Height - 2, (exec_round_button->Height / 4));
          TFT_Set_Font(exec_round_button->FontName, exec_round_button->Font_Color, FO_HORIZONTAL);
          TFT_Write_Text_Return_Pos(exec_round_button->Caption, exec_round_button->Left, exec_round_button->Top);
          TFT_Write_Text(exec_round_button->Caption, (exec_round_button->Left + ((exec_round_button->Width - caption_length) / 2)),
                        (exec_round_button->Top + ((exec_round_button->Height - caption_height) / 2)));
        }
        break;
      }
      break;
    }
    // Box
    case 6: {
      if (PressedObject != 0) {
        exec_box = (TBox*)PressedObject;
        if (exec_box->PressColEnabled == 1) {
          TFT_Set_Brush(exec_box->Transparent, exec_box->Color, exec_box->Gradient, exec_box->Gradient_Orientation, exec_box->Gradient_Start_Color, exec_box->Gradient_End_Color);
          TFT_Set_Pen(exec_box->Pen_Color, exec_box->Pen_Width);
          TFT_Rectangle(exec_box->Left, exec_box->Top, exec_box->Left + exec_box->Width - 1, exec_box->Top + exec_box->Height - 1);
        }
        break;
      }
      break;
    }
  }

  button_order        = -1;
  round_button_order  = -1;
  label_order         = -1;
  box_order           = -1;

  // Buttons
  for ( i = 0 ; i < CurrentScreen->ButtonsCount ; i++ ) {
    local_button = GetButton(i);
    if (local_button->Active == 1) {
      if (IsInsideObject(X, Y, local_button->Left, local_button->Top,
                         local_button->Width, local_button->Height) == 1) {
        button_order = local_button->Order;
        exec_button = local_button;
      }
    }
  }

  // Buttons with Round Edges
  for ( i = 0 ; i < CurrentScreen->Buttons_RoundCount ; i++ ) {
    local_round_button = GetRoundButton(i);
    if (local_round_button->Active == 1) {
      if (IsInsideObject(X, Y, local_round_button->Left, local_round_button->Top,
                         local_round_button->Width, local_round_button->Height) == 1) {
        round_button_order = local_round_button->Order;
        exec_round_button = local_round_button;
      }
    }
  }

  // Labels
  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  // Boxes
  for ( i = 0 ; i < CurrentScreen->BoxesCount ; i++ ) {
    local_box = GetBox(i);
    if (local_box->Active == 1) {
      if (IsInsideObject(X, Y, local_box->Left, local_box->Top,
                         local_box->Width, local_box->Height) == 1) {
        box_order = local_box->Order;
        exec_box = local_box;
      }
    }
  }

  i = -1;
  if (button_order > i)
    i = button_order;
  if (round_button_order > i)
    i = round_button_order;
  if (label_order > i)
    i = label_order;
  if (box_order > i)
    i = box_order;

  if (i != -1) {
  // Buttons
    if (i == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->OnUpPtr != 0)
          exec_button->OnUpPtr();
        if (PressedObject == (void *)exec_button)
          if (exec_button->OnClickPtr != 0)
            exec_button->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Buttons with Round Edges
    if (i == round_button_order) {
      if (exec_round_button->Active == 1) {
        if (exec_round_button->OnUpPtr != 0)
          exec_round_button->OnUpPtr();
        if (PressedObject == (void *)exec_round_button)
          if (exec_round_button->OnClickPtr != 0)
            exec_round_button->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Labels
    if (i == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnUpPtr != 0)
          exec_label->OnUpPtr();
        if (PressedObject == (void *)exec_label)
          if (exec_label->OnClickPtr != 0)
            exec_label->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Boxes
    if (i == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->OnUpPtr != 0)
          exec_box->OnUpPtr();
        if (PressedObject == (void *)exec_box)
          if (exec_box->OnClickPtr != 0)
            exec_box->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

static void Process_TP_Down(unsigned int X, unsigned int Y) {
  int i;
  TButton *local_button;
  TButton *exec_button;
  short button_order;
  TButton_Round *local_round_button;
  TButton_Round *exec_round_button;
  short round_button_order;
  TLabel *local_label;
  TLabel *exec_label;
  short label_order;
  TBox *local_box;
  TBox *exec_box;
  short box_order;

  button_order        = -1;
  round_button_order  = -1;
  label_order         = -1;
  box_order           = -1;

  // Buttons
  for ( i = 0 ; i < CurrentScreen->ButtonsCount ; i++ ) {
    local_button = GetButton(i);
    if (local_button->Active == 1) {
      if (IsInsideObject(X, Y, local_button->Left, local_button->Top,
                         local_button->Width, local_button->Height) == 1) {
        button_order = local_button->Order;
        exec_button = local_button;
      }
    }
  }

  // Buttons with Round Edges
  for ( i = 0 ; i < CurrentScreen->Buttons_RoundCount ; i++ ) {
    local_round_button = GetRoundButton(i);
    if (local_round_button->Active == 1) {
      if (IsInsideObject(X, Y, local_round_button->Left, local_round_button->Top,
                         local_round_button->Width, local_round_button->Height) == 1) {
        round_button_order = local_round_button->Order;
        exec_round_button = local_round_button;
      }
    }
  }

  // Labels
  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  // Boxes
  for ( i = 0 ; i < CurrentScreen->BoxesCount ; i++ ) {
    local_box = GetBox(i);
    if (local_box->Active == 1) {
      if (IsInsideObject(X, Y, local_box->Left, local_box->Top,
                         local_box->Width, local_box->Height) == 1) {
        box_order = local_box->Order;
        exec_box = local_box;
      }
    }
  }

  i = -1;
  if (button_order > i)
    i = button_order;
  if (round_button_order > i)
    i = round_button_order;
  if (label_order > i)
    i = label_order;
  if (box_order > i)
    i = box_order;
  if (i != -1) {
    if (i == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->PressColEnabled == 1) {
          TFT_Set_Brush(exec_button->Transparent, exec_button->Press_Color, exec_button->Gradient, exec_button->Gradient_Orientation, exec_button->Gradient_End_Color, exec_button->Gradient_Start_Color);
          TFT_Set_Pen(exec_button->Pen_Color, exec_button->Pen_Width);
          TFT_Rectangle(exec_button->Left, exec_button->Top, exec_button->Left + exec_button->Width - 1, exec_button->Top + exec_button->Height - 1);
          TFT_Set_Font(exec_button->FontName, exec_button->Font_Color, FO_HORIZONTAL);
          TFT_Write_Text_Return_Pos(exec_button->Caption, exec_button->Left, exec_button->Top);
          TFT_Write_Text(exec_button->Caption, (exec_button->Left + ((exec_button->Width - caption_length) / 2)), (exec_button->Top + ((exec_button->Height - caption_height) / 2)));
        }
        PressedObject = (void *)exec_button;
        PressedObjectType = 0;
        if (exec_button->OnDownPtr != 0) {
          exec_button->OnDownPtr();
          return;
        }
      }
    }

    if (i == round_button_order) {
      if (exec_round_button->Active == 1) {
        if (exec_round_button->PressColEnabled == 1) {
          TFT_Set_Brush(exec_round_button->Transparent, exec_round_button->Press_Color, exec_round_button->Gradient, exec_round_button->Gradient_Orientation,
                        exec_round_button->Gradient_End_Color, exec_round_button->Gradient_Start_Color);
          TFT_Set_Pen(exec_round_button->Pen_Color, exec_round_button->Pen_Width);
          if (exec_round_button->Height > exec_round_button->Width)
            TFT_Rectangle_Round_Edges(exec_round_button->Left + 1, exec_round_button->Top + 1,
                                      exec_round_button->Left + exec_round_button->Width - 2,
                                      exec_round_button->Top + exec_round_button->Height - 2, (exec_round_button->Width / 4));
          else if (exec_round_button->Width > exec_round_button->Height)
            TFT_Rectangle_Round_Edges(exec_round_button->Left + 1, exec_round_button->Top + 1,
                                      exec_round_button->Left + exec_round_button->Width - 2,
                                      exec_round_button->Top + exec_round_button->Height - 2, (exec_round_button->Height / 4));
          TFT_Set_Font(exec_round_button->FontName, exec_round_button->Font_Color, FO_HORIZONTAL);
          TFT_Write_Text_Return_Pos(exec_round_button->Caption, exec_round_button->Left, exec_round_button->Top);
          TFT_Write_Text(exec_round_button->Caption, (exec_round_button->Left + ((exec_round_button->Width - caption_length) / 2)),
                        (exec_round_button->Top + ((exec_round_button->Height - caption_height) / 2)));
        }
        PressedObject = (void *)exec_round_button;
        PressedObjectType = 1;
        if (exec_round_button->OnDownPtr != 0) {
          exec_round_button->OnDownPtr();
          return;
        }
      }
    }

    if (i == label_order) {
      if (exec_label->Active == 1) {
        PressedObject = (void *)exec_label;
        PressedObjectType = 2;
        if (exec_label->OnDownPtr != 0) {
          exec_label->OnDownPtr();
          return;
        }
      }
    }

    if (i == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->PressColEnabled == 1) {
          TFT_Set_Brush(exec_box->Transparent, exec_box->Press_Color, exec_box->Gradient, exec_box->Gradient_Orientation, exec_box->Gradient_End_Color, exec_box->Gradient_Start_Color);
          TFT_Set_Pen(exec_box->Pen_Color, exec_box->Pen_Width);
          TFT_Rectangle(exec_box->Left, exec_box->Top, exec_box->Left + exec_box->Width - 1, exec_box->Top + exec_box->Height - 1);
        }
        PressedObject = (void *)exec_box;
        PressedObjectType = 6;
        if (exec_box->OnDownPtr != 0) {
          exec_box->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (TP_TFT_Press_Detect()) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
    if (TP_TFT_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Start_TP() {
  InitializeTouchPanel();

  // You can get calibration constants using touch panel calibration example
  TP_TFT_Set_Calibration_Consts(149, 776, 68, 765);    // Set calibration constants

  InitializeObjects();
  display_width = Screen2.Width;
  display_height = Screen2.Height;
  DrawScreen(&Screen2);
}
