/* Project name:
     TFT
 * Copyright:
     (c) MikroElektronika, 2010.
 * Revision History:
     20101126 (TL):
       - initial release
 * Description:
     TFT unit test.
     The JPG picture will be shown on our 320x240 TFT display
 * Test configuration:
     MCU:             dsPIC33FJ128GP710
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70286C.pdf
     Dev.Board:       mikroMMB for dsPIC33 Board
                      http://www.mikroe.com/eng/products/view/586/mikrommb-for-dspic33-board/
     Oscillator:      HS-PLL, 64.00000 MHz
                      ac:TFT_display
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/eng/products/view/230/mikropascal-pro-for-dspic30-33-and-pic24/
 * NOTES:
     - none.
 */

#include <built_in.h>

unsigned short TFT_DataPort at LATD;                         // DATA port
unsigned short TFT_DataPort_Direction at TRISD;              // DATA port

sbit TFT_RST_Direction at TRISC1_bit;
sbit TFT_RST  at LATC1_bit;

sbit TFT_RS_Direction  at TRISB15_bit;
sbit TFT_RS   at LATB15_bit;

sbit TFT_CS_Direction  at TRISC3_bit;
sbit TFT_CS   at LATC3_bit;

sbit TFT_RD_Direction  at TRISD12_bit;
sbit TFT_RD   at LATD12_bit;

sbit TFT_WR_Direction  at TRISD13_bit;
sbit TFT_WR   at LATD13_bit;

/*********************************************************************/
const code far char Image1_dspic33[29372];


char text[] = "HeLlO QWeRtY !@#$%^&*(){}:;";
char i;

// write into index register IR, then send command or data
void Set_Index(unsigned short index) {
  TFT_RS = 0;
  Lo(LATD) = index;
  TFT_WR = 0;
  TFT_WR = 1;
}

// write command
void Write_Command(unsigned short cmd) {
  TFT_RS = 1;
  Lo(LATD) = cmd;
  TFT_WR = 0;
  TFT_WR = 1;
}

// write data
void Write_Data(unsigned int _data) {
  TFT_RS = 1;
  Lo(LATE) = Hi(_data);
  Lo(LATD) = Lo(_data);
  TFT_WR = 0;
  TFT_WR = 1;
}


void main() {
  // Setup PLL
  CLKDIVbits.PLLPRE = 0;
  PLLFBD = 38;
  CLKDIVbits.PLLPOST = 0;
  // End Setup PLL

  AD1PCFGL = 0xFFFF;
  AD1PCFGH = 0xFFFF;

  TRISE = 0;
  TRISD = 0;
  Delay_ms(150);
  TFT_Set_Active(Set_Index,Write_Command,Write_Data);
  TFT_Init(320, 240);

  TFT_Fill_Screen(CL_BLACK);
  TFT_Image_Jpeg(0, 0, Image1_dspic33);
  Delay_ms(1000);
}