#include "Paint_objects.h"
#include "Paint_resources.h"
#include "built_in.h"

// TFT module connections
char TFT_DataPort at LATD;
char TFT_DataPort_Direction at TRISD;
sbit TFT_RST at LATC1_bit;
sbit TFT_BLED at LATC2_bit;
sbit TFT_RS at LATB15_bit;
sbit TFT_CS at LATC3_bit;
sbit TFT_RD at LATD12_bit;
sbit TFT_WR at LATD13_bit;
sbit TFT_RST_Direction at TRISC1_bit;
sbit TFT_BLED_Direction at TRISC2_bit;
sbit TFT_RS_Direction at TRISB15_bit;
sbit TFT_CS_Direction at TRISC3_bit;
sbit TFT_RD_Direction at TRISD12_bit;
sbit TFT_WR_Direction at TRISD13_bit;
// End Glcd module connections

// Touch Panel module connections
sbit DriveX_Left at LATB13_bit;
sbit DriveX_Right at LATB11_bit;
sbit DriveY_Up at LATB12_bit;
sbit DriveY_Down at LATB10_bit;
sbit DriveX_Left_Direction at TRISB13_bit;
sbit DriveX_Right_Direction at TRISB11_bit;
sbit DriveY_Up_Direction at TRISB12_bit;
sbit DriveY_Down_Direction at TRISB10_bit;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
char PenDown;
void *PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

// set index
void Set_Index(unsigned short index) {
  TFT_RS = 0;
  Lo(LATD) = index;
  TFT_WR = 0;
  TFT_WR = 1;
}

// write command
void Write_Command(unsigned short cmd) {
  TFT_RS = 1;
  Lo(LATD) = cmd;
  TFT_WR = 0;
  TFT_WR = 1;
}

// write data
void Write_Data(unsigned int _data) {
  TFT_RS = 1;
  Lo(LATE) = Hi(_data);
  Lo(LATD) = Lo(_data);
  TFT_WR = 0;
  TFT_WR = 1;
}

static void InitializeTouchPanel() {
  AD1PCFGL = 0xCFFF;
  AD1PCFGH = 0xCFFF;
  TRISE = 0;
  TRISD = 0;
  CLKDIVbits.PLLPRE = 0;
  PLLFBD = 0x30;
  CLKDIVbits.PLLPOST = 1;
  Delay_ms(150);
  ADC1_Init();
  TFT_Set_Active(Set_Index,Write_Command,Write_Data);
  TFT_Init(320, 240);

  TP_TFT_Init(320, 240, 13, 12);                                  // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(900);                              // Set touch panel ADC threshold

  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}


/////////////////////////
  TScreen *CurrentScreen;

  TScreen                Screen1;
  TBox                   Box1;
  TImage                 Image1;
  TImage                 SelectPen;
  TImage                 SelectEraser;
  TImage                 BackgroundColor;
  TLine                  Line1;
  TLine                  Line2;
  TBox                   Box2;
  TBox                   Box3;
  TBox                   Box4;
  TBox                   Box5;
  TBox                   Box6;
  TBox                   Box7;
  TBox                   Box8;
  TBox                   Box9;
  TBox                   Box10;
  TBox                   Box11;
  TBox                   Box12;
  TBox                   Pen1;
  TBox                   Pen2;
  TBox                   Pen3;
  TBox                   Pen4;
  TImage                 *Screen0_Images[4];
  TBox                   *Screen0_Boxes[16];
  TLine                  *Screen0_Lines[2];




static void InitializeObjects() {
  Screen1.Color                     = 0xC618;
  Screen1.Width                     = 320;
  Screen1.Height                    = 240;
  Screen1.ImagesCount               = 4;
  Screen1.Images                    = Screen0_Images;
  Screen1.BoxesCount                = 16;
  Screen1.Boxes                     = Screen0_Boxes;
  Screen1.LinesCount                = 2;
  Screen1.Lines                     = Screen0_Lines;
  Screen1.ObjectsCount              = 22;
  Screen0_Boxes[0]                  = &Box1;
  Screen0_Images[0]                 = &Image1;
  Screen0_Images[1]                 = &SelectPen;
  Screen0_Images[2]                 = &SelectEraser;
  Screen0_Images[3]                 = &BackgroundColor;
  Screen0_Lines[0]                  = &Line1;
  Screen0_Lines[1]                  = &Line2;
  Screen0_Boxes[1]                  = &Box2;
  Screen0_Boxes[2]                  = &Box3;
  Screen0_Boxes[3]                  = &Box4;
  Screen0_Boxes[4]                  = &Box5;
  Screen0_Boxes[5]                  = &Box6;
  Screen0_Boxes[6]                  = &Box7;
  Screen0_Boxes[7]                  = &Box8;
  Screen0_Boxes[8]                  = &Box9;
  Screen0_Boxes[9]                  = &Box10;
  Screen0_Boxes[10]                 = &Box11;
  Screen0_Boxes[11]                 = &Box12;
  Screen0_Boxes[12]                 = &Pen1;
  Screen0_Boxes[13]                 = &Pen2;
  Screen0_Boxes[14]                 = &Pen3;
  Screen0_Boxes[15]                 = &Pen4;


  Box1.OwnerScreen     = &Screen1;
  Box1.Order           = 0;
  Box1.Left            = 28;
  Box1.Top             = 28;
  Box1.Width           = 295;
  Box1.Height          = 216;
  Box1.Pen_Width       = 1;
  Box1.Pen_Color       = 0x0000;
  Box1.Visible         = 1;
  Box1.Active          = 1;
  Box1.Transparent     = 1;
  Box1.Gradient        = 0;
  Box1.Gradient_Orientation    = 0;
  Box1.Gradient_Start_Color    = 0xFFFF;
  Box1.Gradient_End_Color      = 0xC618;
  Box1.Color           = 0xFFFF;
  Box1.PressColEnabled = 0;
  Box1.Press_Color     = 0x8410;
  Box1.OnUpPtr        = 0;
  Box1.OnDownPtr      = 0;
  Box1.OnClickPtr     = 0;
  Box1.OnPressPtr     = Box1Press;

  Image1.OwnerScreen    = &Screen1;
  Image1.Order          = 1;
  Image1.Left           = 7;
  Image1.Top            = 9;
  Image1.Width          = 14;
  Image1.Height         = 12;
  Image1.Picture_Type   = 0;
  Image1.Picture_Ratio  = 1;
  Image1.Picture_Name   = Image1_New;
  Image1.Visible        = 1;
  Image1.Active         = 1;
  Image1.OnUpPtr        = 0;
  Image1.OnDownPtr      = 0;
  Image1.OnClickPtr     = Image1Click;
  Image1.OnPressPtr     = 0;

  SelectPen.OwnerScreen    = &Screen1;
  SelectPen.Order          = 2;
  SelectPen.Left           = 8;
  SelectPen.Top            = 37;
  SelectPen.Width          = 13;
  SelectPen.Height         = 20;
  SelectPen.Picture_Type   = 0;
  SelectPen.Picture_Ratio  = 1;
  SelectPen.Picture_Name   = SelectPen_Pen;
  SelectPen.Visible        = 1;
  SelectPen.Active         = 1;
  SelectPen.OnUpPtr        = 0;
  SelectPen.OnDownPtr      = 0;
  SelectPen.OnClickPtr     = 0;
  SelectPen.OnPressPtr     = SelectPenPress;

  SelectEraser.OwnerScreen    = &Screen1;
  SelectEraser.Order          = 3;
  SelectEraser.Left           = 4;
  SelectEraser.Top            = 154;
  SelectEraser.Width          = 19;
  SelectEraser.Height         = 14;
  SelectEraser.Picture_Type   = 0;
  SelectEraser.Picture_Ratio  = 1;
  SelectEraser.Picture_Name   = SelectEraser_Eraser;
  SelectEraser.Visible        = 1;
  SelectEraser.Active         = 1;
  SelectEraser.OnUpPtr        = 0;
  SelectEraser.OnDownPtr      = 0;
  SelectEraser.OnClickPtr     = 0;
  SelectEraser.OnPressPtr     = SelectEraserPress;

  BackgroundColor.OwnerScreen    = &Screen1;
  BackgroundColor.Order          = 4;
  BackgroundColor.Left           = 4;
  BackgroundColor.Top            = 177;
  BackgroundColor.Width          = 19;
  BackgroundColor.Height         = 18;
  BackgroundColor.Picture_Type   = 0;
  BackgroundColor.Picture_Ratio  = 1;
  BackgroundColor.Picture_Name   = BackgroundColor_paint;
  BackgroundColor.Visible        = 1;
  BackgroundColor.Active         = 1;
  BackgroundColor.OnUpPtr        = 0;
  BackgroundColor.OnDownPtr      = 0;
  BackgroundColor.OnClickPtr     = 0;
  BackgroundColor.OnPressPtr     = BackgroundColorPress;

  Line1.OwnerScreen    = &Screen1;
  Line1.Order          = 5;
  Line1.First_Point_X  = 4;
  Line1.First_Point_Y  = 28;
  Line1.Second_Point_X = 24;
  Line1.Second_Point_Y = 28;
  Line1.Pen_Width      = 1;
  Line1.Visible        = 1;
  Line1.Color          = 0x0000;

  Line2.OwnerScreen    = &Screen1;
  Line2.Order          = 6;
  Line2.First_Point_X  = 4;
  Line2.First_Point_Y  = 144;
  Line2.Second_Point_X = 24;
  Line2.Second_Point_Y = 144;
  Line2.Pen_Width      = 1;
  Line2.Visible        = 1;
  Line2.Color          = 0x0000;

  Box2.OwnerScreen     = &Screen1;
  Box2.Order           = 7;
  Box2.Left            = 28;
  Box2.Top             = 4;
  Box2.Width           = 20;
  Box2.Height          = 20;
  Box2.Pen_Width       = 3;
  Box2.Pen_Color       = 0xC618;
  Box2.Visible         = 1;
  Box2.Active          = 1;
  Box2.Transparent     = 1;
  Box2.Gradient        = 0;
  Box2.Gradient_Orientation    = 0;
  Box2.Gradient_Start_Color    = 0xFFFF;
  Box2.Gradient_End_Color      = 0xC618;
  Box2.Color           = 0xFFFF;
  Box2.PressColEnabled = 0;
  Box2.Press_Color     = 0x8410;
  Box2.OnUpPtr        = 0;
  Box2.OnDownPtr      = 0;
  Box2.OnClickPtr     = 0;
  Box2.OnPressPtr     = Box2Press;

  Box3.OwnerScreen     = &Screen1;
  Box3.Order           = 8;
  Box3.Left            = 55;
  Box3.Top             = 4;
  Box3.Width           = 20;
  Box3.Height          = 20;
  Box3.Pen_Width       = 3;
  Box3.Pen_Color       = 0xC618;
  Box3.Visible         = 1;
  Box3.Active          = 1;
  Box3.Transparent     = 1;
  Box3.Gradient        = 0;
  Box3.Gradient_Orientation    = 0;
  Box3.Gradient_Start_Color    = 0xFFFF;
  Box3.Gradient_End_Color      = 0xC618;
  Box3.Color           = 0x0000;
  Box3.PressColEnabled = 0;
  Box3.Press_Color     = 0x8410;
  Box3.OnUpPtr        = 0;
  Box3.OnDownPtr      = 0;
  Box3.OnClickPtr     = 0;
  Box3.OnPressPtr     = Box3Press;

  Box4.OwnerScreen     = &Screen1;
  Box4.Order           = 9;
  Box4.Left            = 82;
  Box4.Top             = 4;
  Box4.Width           = 20;
  Box4.Height          = 20;
  Box4.Pen_Width       = 3;
  Box4.Pen_Color       = 0xC618;
  Box4.Visible         = 1;
  Box4.Active          = 1;
  Box4.Transparent     = 1;
  Box4.Gradient        = 0;
  Box4.Gradient_Orientation    = 0;
  Box4.Gradient_Start_Color    = 0xFFFF;
  Box4.Gradient_End_Color      = 0xC618;
  Box4.Color           = 0xF81F;
  Box4.PressColEnabled = 0;
  Box4.Press_Color     = 0x8410;
  Box4.OnUpPtr        = 0;
  Box4.OnDownPtr      = 0;
  Box4.OnClickPtr     = 0;
  Box4.OnPressPtr     = Box4Press;

  Box5.OwnerScreen     = &Screen1;
  Box5.Order           = 10;
  Box5.Left            = 109;
  Box5.Top             = 4;
  Box5.Width           = 20;
  Box5.Height          = 20;
  Box5.Pen_Width       = 3;
  Box5.Pen_Color       = 0xC618;
  Box5.Visible         = 1;
  Box5.Active          = 1;
  Box5.Transparent     = 1;
  Box5.Gradient        = 0;
  Box5.Gradient_Orientation    = 0;
  Box5.Gradient_Start_Color    = 0xFFFF;
  Box5.Gradient_End_Color      = 0xC618;
  Box5.Color           = 0xF800;
  Box5.PressColEnabled = 0;
  Box5.Press_Color     = 0x8410;
  Box5.OnUpPtr        = 0;
  Box5.OnDownPtr      = 0;
  Box5.OnClickPtr     = 0;
  Box5.OnPressPtr     = Box5Press;

  Box6.OwnerScreen     = &Screen1;
  Box6.Order           = 11;
  Box6.Left            = 136;
  Box6.Top             = 4;
  Box6.Width           = 20;
  Box6.Height          = 20;
  Box6.Pen_Width       = 3;
  Box6.Pen_Color       = 0xC618;
  Box6.Visible         = 1;
  Box6.Active          = 1;
  Box6.Transparent     = 1;
  Box6.Gradient        = 0;
  Box6.Gradient_Orientation    = 0;
  Box6.Gradient_Start_Color    = 0xFFFF;
  Box6.Gradient_End_Color      = 0xC618;
  Box6.Color           = 0xFC08;
  Box6.PressColEnabled = 0;
  Box6.Press_Color     = 0x8410;
  Box6.OnUpPtr        = 0;
  Box6.OnDownPtr      = 0;
  Box6.OnClickPtr     = 0;
  Box6.OnPressPtr     = Box6Press;

  Box7.OwnerScreen     = &Screen1;
  Box7.Order           = 12;
  Box7.Left            = 163;
  Box7.Top             = 4;
  Box7.Width           = 20;
  Box7.Height          = 20;
  Box7.Pen_Width       = 3;
  Box7.Pen_Color       = 0xC618;
  Box7.Visible         = 1;
  Box7.Active          = 1;
  Box7.Transparent     = 1;
  Box7.Gradient        = 0;
  Box7.Gradient_Orientation    = 0;
  Box7.Gradient_Start_Color    = 0xFFFF;
  Box7.Gradient_End_Color      = 0xC618;
  Box7.Color           = 0xFFE0;
  Box7.PressColEnabled = 0;
  Box7.Press_Color     = 0x8410;
  Box7.OnUpPtr        = 0;
  Box7.OnDownPtr      = 0;
  Box7.OnClickPtr     = 0;
  Box7.OnPressPtr     = Box7Press;

  Box8.OwnerScreen     = &Screen1;
  Box8.Order           = 13;
  Box8.Left            = 189;
  Box8.Top             = 4;
  Box8.Width           = 20;
  Box8.Height          = 20;
  Box8.Pen_Width       = 3;
  Box8.Pen_Color       = 0xC618;
  Box8.Visible         = 1;
  Box8.Active          = 1;
  Box8.Transparent     = 1;
  Box8.Gradient        = 0;
  Box8.Gradient_Orientation    = 0;
  Box8.Gradient_Start_Color    = 0xFFFF;
  Box8.Gradient_End_Color      = 0xC618;
  Box8.Color           = 0x87E0;
  Box8.PressColEnabled = 0;
  Box8.Press_Color     = 0x8410;
  Box8.OnUpPtr        = 0;
  Box8.OnDownPtr      = 0;
  Box8.OnClickPtr     = 0;
  Box8.OnPressPtr     = Box8Press;

  Box9.OwnerScreen     = &Screen1;
  Box9.Order           = 14;
  Box9.Left            = 216;
  Box9.Top             = 4;
  Box9.Width           = 20;
  Box9.Height          = 20;
  Box9.Pen_Width       = 3;
  Box9.Pen_Color       = 0xC618;
  Box9.Visible         = 1;
  Box9.Active          = 1;
  Box9.Transparent     = 1;
  Box9.Gradient        = 0;
  Box9.Gradient_Orientation    = 0;
  Box9.Gradient_Start_Color    = 0xFFFF;
  Box9.Gradient_End_Color      = 0xC618;
  Box9.Color           = 0x07FF;
  Box9.PressColEnabled = 0;
  Box9.Press_Color     = 0x8410;
  Box9.OnUpPtr        = 0;
  Box9.OnDownPtr      = 0;
  Box9.OnClickPtr     = 0;
  Box9.OnPressPtr     = Box9Press;

  Box10.OwnerScreen     = &Screen1;
  Box10.Order           = 15;
  Box10.Left            = 243;
  Box10.Top             = 4;
  Box10.Width           = 20;
  Box10.Height          = 20;
  Box10.Pen_Width       = 3;
  Box10.Pen_Color       = 0xC618;
  Box10.Visible         = 1;
  Box10.Active          = 1;
  Box10.Transparent     = 1;
  Box10.Gradient        = 0;
  Box10.Gradient_Orientation    = 0;
  Box10.Gradient_Start_Color    = 0xFFFF;
  Box10.Gradient_End_Color      = 0xC618;
  Box10.Color           = 0x041F;
  Box10.PressColEnabled = 0;
  Box10.Press_Color     = 0x8410;
  Box10.OnUpPtr        = 0;
  Box10.OnDownPtr      = 0;
  Box10.OnClickPtr     = 0;
  Box10.OnPressPtr     = Box10Press;

  Box11.OwnerScreen     = &Screen1;
  Box11.Order           = 16;
  Box11.Left            = 270;
  Box11.Top             = 4;
  Box11.Width           = 20;
  Box11.Height          = 20;
  Box11.Pen_Width       = 3;
  Box11.Pen_Color       = 0xC618;
  Box11.Visible         = 1;
  Box11.Active          = 1;
  Box11.Transparent     = 1;
  Box11.Gradient        = 0;
  Box11.Gradient_Orientation    = 0;
  Box11.Gradient_Start_Color    = 0xFFFF;
  Box11.Gradient_End_Color      = 0xC618;
  Box11.Color           = 0x801F;
  Box11.PressColEnabled = 0;
  Box11.Press_Color     = 0x8410;
  Box11.OnUpPtr        = 0;
  Box11.OnDownPtr      = 0;
  Box11.OnClickPtr     = 0;
  Box11.OnPressPtr     = Box11Press;

  Box12.OwnerScreen     = &Screen1;
  Box12.Order           = 17;
  Box12.Left            = 297;
  Box12.Top             = 4;
  Box12.Width           = 20;
  Box12.Height          = 20;
  Box12.Pen_Width       = 3;
  Box12.Pen_Color       = 0xC618;
  Box12.Visible         = 1;
  Box12.Active          = 1;
  Box12.Transparent     = 1;
  Box12.Gradient        = 0;
  Box12.Gradient_Orientation    = 0;
  Box12.Gradient_Start_Color    = 0xFFFF;
  Box12.Gradient_End_Color      = 0xC618;
  Box12.Color           = 0x4010;
  Box12.PressColEnabled = 0;
  Box12.Press_Color     = 0x8410;
  Box12.OnUpPtr        = 0;
  Box12.OnDownPtr      = 0;
  Box12.OnClickPtr     = 0;
  Box12.OnPressPtr     = Box12Press;

  Pen1.OwnerScreen     = &Screen1;
  Pen1.Order           = 18;
  Pen1.Left            = 4;
  Pen1.Top             = 69;
  Pen1.Width           = 18;
  Pen1.Height          = 18;
  Pen1.Pen_Width       = 3;
  Pen1.Pen_Color       = 0xC618;
  Pen1.Visible         = 1;
  Pen1.Active          = 1;
  Pen1.Transparent     = 1;
  Pen1.Gradient        = 0;
  Pen1.Gradient_Orientation    = 0;
  Pen1.Gradient_Start_Color    = 0xFFFF;
  Pen1.Gradient_End_Color      = 0xC618;
  Pen1.Color           = 0xFFFF;
  Pen1.PressColEnabled = 0;
  Pen1.Press_Color     = 0x8410;
  Pen1.OnUpPtr        = 0;
  Pen1.OnDownPtr      = 0;
  Pen1.OnClickPtr     = 0;
  Pen1.OnPressPtr     = Pen1Press;

  Pen2.OwnerScreen     = &Screen1;
  Pen2.Order           = 19;
  Pen2.Left            = 4;
  Pen2.Top             = 98;
  Pen2.Width           = 13;
  Pen2.Height          = 13;
  Pen2.Pen_Width       = 3;
  Pen2.Pen_Color       = 0xC618;
  Pen2.Visible         = 1;
  Pen2.Active          = 1;
  Pen2.Transparent     = 1;
  Pen2.Gradient        = 0;
  Pen2.Gradient_Orientation    = 0;
  Pen2.Gradient_Start_Color    = 0xFFFF;
  Pen2.Gradient_End_Color      = 0xC618;
  Pen2.Color           = 0xFFFF;
  Pen2.PressColEnabled = 0;
  Pen2.Press_Color     = 0x8410;
  Pen2.OnUpPtr        = 0;
  Pen2.OnDownPtr      = 0;
  Pen2.OnClickPtr     = 0;
  Pen2.OnPressPtr     = Pen2Press;

  Pen3.OwnerScreen     = &Screen1;
  Pen3.Order           = 20;
  Pen3.Left            = 4;
  Pen3.Top             = 123;
  Pen3.Width           = 9;
  Pen3.Height          = 9;
  Pen3.Pen_Width       = 3;
  Pen3.Pen_Color       = 0xC618;
  Pen3.Visible         = 1;
  Pen3.Active          = 1;
  Pen3.Transparent     = 1;
  Pen3.Gradient        = 0;
  Pen3.Gradient_Orientation    = 0;
  Pen3.Gradient_Start_Color    = 0xFFFF;
  Pen3.Gradient_End_Color      = 0xC618;
  Pen3.Color           = 0xFFFF;
  Pen3.PressColEnabled = 0;
  Pen3.Press_Color     = 0x8410;
  Pen3.OnUpPtr        = 0;
  Pen3.OnDownPtr      = 0;
  Pen3.OnClickPtr     = 0;
  Pen3.OnPressPtr     = Pen3Press;

  Pen4.OwnerScreen     = &Screen1;
  Pen4.Order           = 21;
  Pen4.Left            = 4;
  Pen4.Top             = 206;
  Pen4.Width           = 18;
  Pen4.Height          = 18;
  Pen4.Pen_Width       = 3;
  Pen4.Pen_Color       = 0xC618;
  Pen4.Visible         = 1;
  Pen4.Active          = 1;
  Pen4.Transparent     = 1;
  Pen4.Gradient        = 0;
  Pen4.Gradient_Orientation    = 0;
  Pen4.Gradient_Start_Color    = 0xFFFF;
  Pen4.Gradient_End_Color      = 0xC618;
  Pen4.Color           = 0xFFFF;
  Pen4.PressColEnabled = 0;
  Pen4.Press_Color     = 0x8410;
  Pen4.OnUpPtr        = 0;
  Pen4.OnDownPtr      = 0;
  Pen4.OnClickPtr     = 0;
  Pen4.OnPressPtr     = Pen4Press;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetImage(index)               CurrentScreen->Images[index]
#define GetBox(index)                 CurrentScreen->Boxes[index]
#define GetLine(index)                CurrentScreen->Lines[index]


void DrawImage(TImage *Aimage) {
  if (Aimage->Visible == 1) {
    if (Aimage->Picture_Type == 0) {
      TFT_Image(Aimage->Left, Aimage->Top, Aimage->Picture_Name, Aimage->Picture_Ratio);
    }
    if (Aimage->Picture_Type == 1) {
      TFT_Image_Jpeg(Aimage->Left, Aimage->Top, Aimage->Picture_Name);
    }
  }
}

static void DrawImages() {
  int i;
  TImage *local_image;

  for ( i = 0 ; i < CurrentScreen->ImagesCount ; i++ ) {
    local_image = GetImage(i);
    DrawImage(local_image);
  }
}


void DrawBox(TBox *ABox) {
  if (ABox->Visible == 1) {
    TFT_Set_Brush(ABox->Transparent, ABox->Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_Start_Color, ABox->Gradient_End_Color);
    TFT_Set_Pen(ABox->Pen_Color, ABox->Pen_Width);
    TFT_Rectangle(ABox->Left, ABox->Top, ABox->Left + ABox->Width - 1, ABox->Top + ABox->Height - 1);
  }
}

static void DrawBoxes() {
  int i;
  TBox *local_box;

  for ( i = 0 ; i < CurrentScreen->BoxesCount ; i++ ) {
    local_box = GetBox(i);
    DrawBox(local_box);
  }
}


void DrawLine(TLine *Aline) {
  if (Aline->Visible == 1) {
    TFT_Set_Pen(Aline->Color, Aline->Pen_Width);
    TFT_Line(Aline->First_Point_X, Aline->First_Point_Y, Aline->Second_Point_X, Aline->Second_Point_Y);
  }
}

static void DrawLines() {
  int i;
  TLine *local_line;

  for ( i = 0 ; i < CurrentScreen->LinesCount ; i++ ) {
    local_line = GetLine(i);
    DrawLine(local_line);
  }
}

void DrawScreen(TScreen *aScreen) {
  unsigned short order;
  unsigned short image_idx;
  TImage *local_image;
  unsigned short box_idx;
  TBox *local_box;
  unsigned short line_idx;
  TLine *local_line;
  order = 0;
  image_idx = 0;
  box_idx = 0;
  line_idx = 0;

  if ((display_width != aScreen->Width) || (display_height != aScreen->Height)) {
    TFT_BLED_Direction = 0;
    TFT_BLED           = 0;
    TFT_Init(aScreen->Width, aScreen->Height);
    TP_TFT_Init(aScreen->Width, aScreen->Height, 13, 12);                                  // Initialize touch panel
    TP_TFT_Set_ADC_Threshold(900);                              // Set touch panel ADC threshold
    TFT_Fill_Screen(aScreen->Color);
    display_width = aScreen->Width;
    display_height = aScreen->Height;
    TFT_BLED           = 1;
  }
  else
    TFT_Fill_Screen(aScreen->Color);

  CurrentScreen = aScreen;


  while (order < CurrentScreen->ObjectsCount) {
    if (box_idx < CurrentScreen->BoxesCount) {
      local_box = GetBox(box_idx);
      if (order == local_box->Order) {
        box_idx++;
        order++;
        DrawBox(local_box);
      }
    }

    if (line_idx  < CurrentScreen->LinesCount) {
      local_line = GetLine(line_idx);
      if (order == local_line->Order) {
        line_idx++;
        order++;
        DrawLine(local_line);
      }
    }

    if (image_idx  < CurrentScreen->ImagesCount) {
      local_image = GetImage(image_idx);
      if (order == local_image->Order) {
        image_idx++;
        order++;
        DrawImage(local_image);
      }
    }

  }
}

static void Process_TP_Press(unsigned int X, unsigned int Y) {
  int i;
  TImage *local_image;
  TImage *exec_image;
  short image_order;
  TBox *local_box;
  TBox *exec_box;
  short box_order;

  image_order         = -1;
  box_order           = -1;

  // Images
  for ( i = 0 ; i < CurrentScreen->ImagesCount ; i++ ) {
    local_image = GetImage(i);
    if (local_image->Active == 1) {
      if (IsInsideObject(X, Y, local_image->Left, local_image->Top,
                         local_image->Width, local_image->Height) == 1) {
        image_order = local_image->Order;
        exec_image = local_image;
      }
    }
  }

  // Boxes
  for ( i = 0 ; i < CurrentScreen->BoxesCount ; i++ ) {
    local_box = GetBox(i);
    if (local_box->Active == 1) {
      if (IsInsideObject(X, Y, local_box->Left, local_box->Top,
                         local_box->Width, local_box->Height) == 1) {
        box_order = local_box->Order;
        exec_box = local_box;
      }
    }
  }

  i = -1;
  if (image_order > i)
    i = image_order;
  if (box_order > i)
    i = box_order;

  if (i != -1) {
    if (i == image_order) {
      if (exec_image->Active == 1) {
        if (exec_image->OnPressPtr != 0) {
          exec_image->OnPressPtr();
          return;
        }
      }
    }

    if (i == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->OnPressPtr != 0) {
          exec_box->OnPressPtr();
          return;
        }
      }
    }

  }
}

static void Process_TP_Up(unsigned int X, unsigned int Y) {
  int i;
  TImage *local_image;
  TImage *exec_image;
  short image_order;
  TBox *local_box;
  TBox *exec_box;
  short box_order;

  switch (PressedObjectType) {
    // Box
    case 6: {
      if (PressedObject != 0) {
        exec_box = (TBox*)PressedObject;
        if (exec_box->PressColEnabled == 1) {
          TFT_Set_Brush(exec_box->Transparent, exec_box->Color, exec_box->Gradient, exec_box->Gradient_Orientation, exec_box->Gradient_Start_Color, exec_box->Gradient_End_Color);
          TFT_Set_Pen(exec_box->Pen_Color, exec_box->Pen_Width);
          TFT_Rectangle(exec_box->Left, exec_box->Top, exec_box->Left + exec_box->Width - 1, exec_box->Top + exec_box->Height - 1);
        }
        break;
      }
      break;
    }
  }

  image_order         = -1;
  box_order           = -1;

  // Images
  for ( i = 0 ; i < CurrentScreen->ImagesCount ; i++ ) {
    local_image = GetImage(i);
    if (local_image->Active == 1) {
      if (IsInsideObject(X, Y, local_image->Left, local_image->Top,
                         local_image->Width, local_image->Height) == 1) {
        image_order = local_image->Order;
        exec_image = local_image;
      }
    }
  }

  // Boxes
  for ( i = 0 ; i < CurrentScreen->BoxesCount ; i++ ) {
    local_box = GetBox(i);
    if (local_box->Active == 1) {
      if (IsInsideObject(X, Y, local_box->Left, local_box->Top,
                         local_box->Width, local_box->Height) == 1) {
        box_order = local_box->Order;
        exec_box = local_box;
      }
    }
  }

  i = -1;
  if (image_order > i)
    i = image_order;
  if (box_order > i)
    i = box_order;

  if (i != -1) {
  // Images
    if (i == image_order) {
      if (exec_image->Active == 1) {
        if (exec_image->OnUpPtr != 0)
          exec_image->OnUpPtr();
        if (PressedObject == (void *)exec_image)
          if (exec_image->OnClickPtr != 0)
            exec_image->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Boxes
    if (i == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->OnUpPtr != 0)
          exec_box->OnUpPtr();
        if (PressedObject == (void *)exec_box)
          if (exec_box->OnClickPtr != 0)
            exec_box->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

static void Process_TP_Down(unsigned int X, unsigned int Y) {
  int i;
  TImage *local_image;
  TImage *exec_image;
  short image_order;
  TBox *local_box;
  TBox *exec_box;
  short box_order;

  image_order         = -1;
  box_order           = -1;

  // Images
  for ( i = 0 ; i < CurrentScreen->ImagesCount ; i++ ) {
    local_image = GetImage(i);
    if (local_image->Active == 1) {
      if (IsInsideObject(X, Y, local_image->Left, local_image->Top,
                         local_image->Width, local_image->Height) == 1) {
        image_order = local_image->Order;
        exec_image = local_image;
      }
    }
  }

  // Boxes
  for ( i = 0 ; i < CurrentScreen->BoxesCount ; i++ ) {
    local_box = GetBox(i);
    if (local_box->Active == 1) {
      if (IsInsideObject(X, Y, local_box->Left, local_box->Top,
                         local_box->Width, local_box->Height) == 1) {
        box_order = local_box->Order;
        exec_box = local_box;
      }
    }
  }

  i = -1;
  if (image_order > i)
    i = image_order;
  if (box_order > i)
    i = box_order;
  if (i != -1) {
    if (i == image_order) {
      if (exec_image->Active == 1) {
        PressedObject = (void *)exec_image;
        PressedObjectType = 3;
        if (exec_image->OnDownPtr != 0) {
          exec_image->OnDownPtr();
          return;
        }
      }
    }

    if (i == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->PressColEnabled == 1) {
          TFT_Set_Brush(exec_box->Transparent, exec_box->Press_Color, exec_box->Gradient, exec_box->Gradient_Orientation, exec_box->Gradient_End_Color, exec_box->Gradient_Start_Color);
          TFT_Set_Pen(exec_box->Pen_Color, exec_box->Pen_Width);
          TFT_Rectangle(exec_box->Left, exec_box->Top, exec_box->Left + exec_box->Width - 1, exec_box->Top + exec_box->Height - 1);
        }
        PressedObject = (void *)exec_box;
        PressedObjectType = 6;
        if (exec_box->OnDownPtr != 0) {
          exec_box->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (TP_TFT_Press_Detect()) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
    if (TP_TFT_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Start_TP() {
  InitializeTouchPanel();

  // You can get calibration constants using touch panel calibration example
  TP_TFT_Set_Calibration_Consts(149, 776, 68, 765);    // Set calibration constants

  InitializeObjects();
  display_width = Screen1.Width;
  display_height = Screen1.Height;
  DrawScreen(&Screen1);
}
