#include "mEPlayer_playlist_manager.h"
#include "mEPlayer_mp3decoder.h"

extern const unsigned int   MAX_FILES_COUNT;

extern const unsigned short PLAYER_PLAYLIST_NOT_SELECTED;
extern const unsigned short PLAYER_PLAYLIST_LOADED_OK;
extern const unsigned short PLAYER_PLAYLIST_NOT_LOADED;
extern const unsigned short PLAYER_FILE_NOT_LOADED;
extern const unsigned short PLAYER_FILE_LOADED_OK;
extern const unsigned short PLAYER_FILE_NOT_FOUND;
extern const unsigned short PLAYER_FAT_INIT_FAILED;
extern const unsigned short PLAYER_STOPPED;
extern const unsigned short PLAYER_PLAY;
extern const unsigned short PLAYER_PAUSED;
extern void DisplayTimeElapsed();
extern void NextClick();
TPlaylist myPlaylist;
TPlaylist *currentPlaylist;

const unsigned int BUFFER_SIZE = 2048;
unsigned char  BufferLarge[BUFFER_SIZE];

const unsigned int TIMER_RESET_VALUE = 0x0BB8;

void TimerInterrupt() iv IVT_ADDR_T1INTERRUPT{
  TMR1 = TIMER_RESET_VALUE;
  T1IF_bit = 0;
  if (currentPlaylist > 0){
    if ((currentPlaylist->PlaylistStatus == PLAYER_PLAYLIST_LOADED_OK) && (currentPlaylist->PlayStatus == PLAYER_PLAY)){
      currentPlaylist->TimeElapsed++;
      DisplayTimeElapsed();
    }
  }

}

void setActivePlaylist(TPlaylist *NewPlaylist){
  currentPlaylist = NewPlaylist;
}

void setInitValues(){
   // Disable other peripheral modules on the same SPI bus
  TRISC4_bit = 0;
  LATC4_bit = 1;   // Disable Serial Flash module

  Delay_ms(150);
  SPI2_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, _SPI_PRESCALE_SEC_1, _SPI_PRESCALE_PRI_4,
                     _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_HIGH, _SPI_ACTIVE_2_IDLE);
  MP3_init();
  currentPlaylist = 0;
  
  // Timer Interrupt every 1s;

  TMR1 = TIMER_RESET_VALUE;
  T1CON = 0b1010000000110000;
  T1IF_bit = 0;
  T1IE_bit = 1;

}

static void addFileToPlaylist(unsigned char* FileName, unsigned char* ArtistName, unsigned char* SongName){
  strcpy(&currentPlaylist->Files[currentPlaylist->FilesCount]->FileName, FileName);
  strcpy(&currentPlaylist->Files[currentPlaylist->FilesCount]->ArtistName, ArtistName);
  strcpy(&currentPlaylist->Files[currentPlaylist->FilesCount]->SongName, SongName);
  currentPlaylist->Files[currentPlaylist->FilesCount]->CurrentPos = 0;
  currentPlaylist->Files[currentPlaylist->FilesCount]->FileSize = 0;
  currentPlaylist->FilesCount++;
}

unsigned short loadPlaylist(){
  currentPlaylist->PlaylistStatus = PLAYER_PLAYLIST_NOT_SELECTED;
  if (currentPlaylist > 0){
    currentPlaylist->CurrentFileIndex = -1;
    currentPlaylist->FilesCount       = 0;
    currentPlaylist->FileStatus       = PLAYER_FILE_NOT_LOADED;
    currentPlaylist->PlayStatus       = PLAYER_STOPPED;
    currentPlaylist->PlaylistStatus   = PLAYER_PLAYLIST_NOT_LOADED;
    currentPlaylist->VolumeLeft       = 250;
    currentPlaylist->VolumeRight      = 250;
    currentPlaylist->Muted            = 0;
    currentPlaylist->TimeElapsed      = 0;

    // Add Files Here **********************************************************
    addFileToPlaylist("sound01.mp3", "Tool", "The Grudge");
    addFileToPlaylist("sound02.mp3", "Tool", "Eon Blue Apocalypse");
    addFileToPlaylist("sound03.mp3", "Tool", "The Patient");
    addFileToPlaylist("sound04.mp3", "Tool", "Mantra");
    addFileToPlaylist("sound05.mp3", "Tool", "Schism");
    addFileToPlaylist("sound06.mp3", "Tool", "Parabol");
    addFileToPlaylist("sound07.mp3", "Tool", "Parabola");
    addFileToPlaylist("sound08.mp3", "Tool", "Ticks & Leeches");
    addFileToPlaylist("sound09.mp3", "Tool", "Lateralus");
    addFileToPlaylist("sound10.mp3", "Tool", "Disposition");
    addFileToPlaylist("sound11.mp3", "Tool", "Reflection");
    addFileToPlaylist("sound12.mp3", "Tool", "Triad");
    addFileToPlaylist("sound13.mp3", "Tool", "Faap De Oiad");
    // End Adding Files ********************************************************
   
    if (currentPlaylist->FilesCount > 0){
      currentPlaylist->CurrentFileIndex = 0;
      currentPlaylist->NowPlaying = &currentPlaylist->Files[currentPlaylist->CurrentFileIndex];
      currentPlaylist->PlaylistStatus = PLAYER_PLAYLIST_LOADED_OK;
    }
  }
  DisplayTimeElapsed();
  return  currentPlaylist->PlaylistStatus;
}

unsigned short loadNext(){
  return 0;
}

unsigned short loadPrevious(){
  return 0;
}

unsigned short loadCurrent(){
  if (Mmc_Fat_Init() == 0) {
    if (Mmc_Fat_Assign(&currentPlaylist->NowPlaying->FileName, 0) ) {
      Mmc_Fat_Reset(&currentPlaylist->NowPlaying->FileSize);
      currentPlaylist->NowPlaying->CurrentPos = 0;
      currentPlaylist->TimeElapsed = 0;
      MP3_reset(currentPlaylist->VolumeLeft, currentPlaylist->VolumeRight);
      currentPlaylist->FileStatus = PLAYER_FILE_LOADED_OK;
    }
    else
      currentPlaylist->FileStatus =  PLAYER_FILE_NOT_FOUND;
  }
  else
    currentPlaylist->FileStatus = PLAYER_FAT_INIT_FAILED;
    
  return currentPlaylist->FileStatus;
}

void playMP3Chunk(){
  unsigned long i;
  if ((currentPlaylist->PlaylistStatus == PLAYER_PLAYLIST_LOADED_OK) && 
      (currentPlaylist->FileStatus     == PLAYER_FILE_LOADED_OK) &&
      (currentPlaylist->PlayStatus     == PLAYER_PLAY)){
    if (currentPlaylist->NowPlaying->CurrentPos != currentPlaylist->NowPlaying->FileSize){
      // If file hasn't reached the end
      if ((currentPlaylist->NowPlaying->FileSize - currentPlaylist->NowPlaying->CurrentPos) > BUFFER_SIZE) {
        // if possible, read and decode BUFFER_SIZE chunk of data
        for (i = 0; i < BUFFER_SIZE; i++) {
          Mmc_Fat_Read(BufferLarge + i);
        }

        //for (i = 0; i < (BUFFER_SIZE >> 6); i++) {
        for (i = 0; i < BUFFER_SIZE/32; i++) {
          MP3_SDI_Write_32(BufferLarge + i*32);
          //MP3_SDI_Write_32(BufferLarge + (i << 6));
        }
        currentPlaylist->NowPlaying->CurrentPos += BUFFER_SIZE;
      }
      else{
        // Send the rest of the file to MP3 SDI
        for (i = 0; i < currentPlaylist->NowPlaying->FileSize - currentPlaylist->NowPlaying->CurrentPos; i++) {
          Mmc_Fat_Read(BufferLarge + i);
        }

        for (i = 0; i < currentPlaylist->NowPlaying->FileSize - currentPlaylist->NowPlaying->CurrentPos; i++) {
          MP3_SDI_Write(BufferLarge[i]);
        }
        currentPlaylist->NowPlaying->CurrentPos = currentPlaylist->NowPlaying->FileSize;
        NextClick();
      }
    }
    else{
      // If file has reached the end
      currentPlaylist->PlayStatus = PLAYER_STOPPED;
      currentPlaylist->NowPlaying->CurrentPos = 0;
    }
  }
}

void playFile(){
  currentPlaylist->PlayStatus = PLAYER_PLAY;
}

void pauseFile(){
  currentPlaylist->PlayStatus = PLAYER_PAUSED;
}

void stopPlaying(){
  currentPlaylist->PlayStatus = PLAYER_STOPPED;
  currentPlaylist->NowPlaying->CurrentPos = 0;
  currentPlaylist->TimeElapsed = 0;
}