#include "mePlayer_objects.h"
#include "mePlayer_resources.h"
#include "built_in.h"

// TFT module connections
char TFT_DataPort at LATD;
char TFT_DataPort_Direction at TRISD;
sbit TFT_RST at LATC1_bit;
sbit TFT_BLED at LATC2_bit;
sbit TFT_RS at LATB15_bit;
sbit TFT_CS at LATC3_bit;
sbit TFT_RD at LATD12_bit;
sbit TFT_WR at LATD13_bit;
sbit TFT_RST_Direction at TRISC1_bit;
sbit TFT_BLED_Direction at TRISC2_bit;
sbit TFT_RS_Direction at TRISB15_bit;
sbit TFT_CS_Direction at TRISC3_bit;
sbit TFT_RD_Direction at TRISD12_bit;
sbit TFT_WR_Direction at TRISD13_bit;
// End Glcd module connections

// Touch Panel module connections
sbit DriveX_Left at LATB13_bit;
sbit DriveX_Right at LATB11_bit;
sbit DriveY_Up at LATB12_bit;
sbit DriveY_Down at LATB10_bit;
sbit DriveX_Left_Direction at TRISB13_bit;
sbit DriveX_Right_Direction at TRISB11_bit;
sbit DriveY_Up_Direction at TRISB12_bit;
sbit DriveY_Down_Direction at TRISB10_bit;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
char PenDown;
void *PressedObject;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

// set index
void Set_Index(unsigned short index) {
  TFT_RS = 0;
  Lo(LATD) = index;
  TFT_WR = 0;
  TFT_WR = 1;
}

// write command
void Write_Command(unsigned short cmd) {
  TFT_RS = 1;
  Lo(LATD) = cmd;
  TFT_WR = 0;
  TFT_WR = 1;
}

// write data
void Write_Data(unsigned int _data) {
  TFT_RS = 1;
  Lo(LATE) = Hi(_data);
  Lo(LATD) = Lo(_data);
  TFT_WR = 0;
  TFT_WR = 1;
}

static void InitializeTouchPanel() {
  AD1PCFGL = 0xCFFF;
  AD1PCFGH = 0xCFFF;
  TRISE = 0;
  TRISD = 0;
  CLKDIVbits.PLLPRE = 0;
  PLLFBD = 0x30;
  CLKDIVbits.PLLPOST = 1;
  Delay_ms(150);
  ADC1_Init();
  TFT_Set_Active(Set_Index,Write_Command,Write_Data);
  TFT_Init(240, 320);

  TP_TFT_Init(240, 320, 13, 12);                                  // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(900);                              // Set touch panel ADC threshold

  PenDown = 0;
  PressedObject = 0;
}


/////////////////////////
  TScreen *CurrentScreen;

  TScreen                Screen1;
  TImage                 Mute;
  TImage                 Pause;
  TImage                 Header;
  TBox                   Box1;
  TImage                 Previous;
  TImage                 Stop;
  TImage                 Next;
  TImage                 Volume;
  TImage                 Play;
  TImage                 Volume_Bar;
  TBox                   VolumeBar;
  TLabel                 Label1;
  char Label1_Caption[7] = "--:--";

  TLabel                 Label2;
  char Label2_Caption[52] = "Please load the playlist";

  TLabel                 Label3;
  char Label3_Caption[52] = "to start playing.";

  TLabel                 *Screen0_Labels[3];
  TImage                 *Screen0_Images[9];
  TBox                   *Screen0_Boxes[2];




static void InitializeObjects() {
  Screen1.Color                     = 0x0000;
  Screen1.Width                     = 240;
  Screen1.Height                    = 320;
  Screen1.LabelsCount               = 3;
  Screen1.Labels                    = Screen0_Labels;
  Screen1.ImagesCount               = 9;
  Screen1.Images                    = Screen0_Images;
  Screen1.BoxesCount                = 2;
  Screen1.Boxes                     = Screen0_Boxes;
  Screen1.ObjectsCount              = 14;
  Screen0_Images[0]                 = &Mute;
  Screen0_Images[1]                 = &Pause;
  Screen0_Images[2]                 = &Header;
  Screen0_Boxes[0]                  = &Box1;
  Screen0_Images[3]                 = &Previous;
  Screen0_Images[4]                 = &Stop;
  Screen0_Images[5]                 = &Next;
  Screen0_Images[6]                 = &Volume;
  Screen0_Images[7]                 = &Play;
  Screen0_Images[8]                 = &Volume_Bar;
  Screen0_Boxes[1]                  = &VolumeBar;
  Screen0_Labels[0]                 = &Label1;
  Screen0_Labels[1]                 = &Label2;
  Screen0_Labels[2]                 = &Label3;


  Mute.OwnerScreen    = &Screen1;
  Mute.Order          = 0;
  Mute.Left           = 124;
  Mute.Top            = 294;
  Mute.Width          = 31;
  Mute.Height         = 26;
  Mute.Picture_Type   = 0;
  Mute.Picture_Ratio  = 1;
  Mute.Picture_Name   = Mute_mute;
  Mute.Visible        = 1;
  Mute.Active         = 1;
  Mute.OnUpPtr        = 0;
  Mute.OnDownPtr      = MuteClick;
  Mute.OnClickPtr     = 0;
  Mute.OnPressPtr     = 0;

  Pause.OwnerScreen    = &Screen1;
  Pause.Order          = 1;
  Pause.Left           = 31;
  Pause.Top            = 294;
  Pause.Width          = 31;
  Pause.Height         = 26;
  Pause.Picture_Type   = 0;
  Pause.Picture_Ratio  = 1;
  Pause.Picture_Name   = Pause_pause;
  Pause.Visible        = 1;
  Pause.Active         = 1;
  Pause.OnUpPtr        = 0;
  Pause.OnDownPtr      = PauseClick;
  Pause.OnClickPtr     = 0;
  Pause.OnPressPtr     = 0;

  Header.OwnerScreen    = &Screen1;
  Header.Order          = 2;
  Header.Left           = 0;
  Header.Top            = 0;
  Header.Width          = 240;
  Header.Height         = 27;
  Header.Picture_Type   = 0;
  Header.Picture_Ratio  = 1;
  Header.Picture_Name   = Header_header;
  Header.Visible        = 1;
  Header.Active         = 1;
  Header.OnUpPtr        = 0;
  Header.OnDownPtr      = HeaderClick;
  Header.OnClickPtr     = 0;
  Header.OnPressPtr     = 0;

  Box1.OwnerScreen     = &Screen1;
  Box1.Order           = 3;
  Box1.Left            = 0;
  Box1.Top             = 48;
  Box1.Width           = 240;
  Box1.Height          = 246;
  Box1.Pen_Width       = 0;
  Box1.Pen_Color       = 0x0000;
  Box1.Visible         = 1;
  Box1.Active          = 1;
  Box1.Transparent     = 1;
  Box1.Gradient        = 1;
  Box1.Gradient_Orientation    = 0;
  Box1.Gradient_Start_Color    = 0x0000;
  Box1.Gradient_End_Color      = 0xF800;
  Box1.Color           = 0xC618;
  Box1.PressColEnabled = 0;
  Box1.Press_Color     = 0x8410;
  Box1.OnUpPtr        = 0;
  Box1.OnDownPtr      = 0;
  Box1.OnClickPtr     = 0;
  Box1.OnPressPtr     = 0;

  Previous.OwnerScreen    = &Screen1;
  Previous.Order          = 4;
  Previous.Left           = 0;
  Previous.Top            = 294;
  Previous.Width          = 31;
  Previous.Height         = 26;
  Previous.Picture_Type   = 0;
  Previous.Picture_Ratio  = 1;
  Previous.Picture_Name   = Previous_prev;
  Previous.Visible        = 1;
  Previous.Active         = 1;
  Previous.OnUpPtr        = 0;
  Previous.OnDownPtr      = PreviousClick;
  Previous.OnClickPtr     = 0;
  Previous.OnPressPtr     = 0;

  Stop.OwnerScreen    = &Screen1;
  Stop.Order          = 5;
  Stop.Left           = 62;
  Stop.Top            = 294;
  Stop.Width          = 31;
  Stop.Height         = 26;
  Stop.Picture_Type   = 0;
  Stop.Picture_Ratio  = 1;
  Stop.Picture_Name   = Stop_stop;
  Stop.Visible        = 1;
  Stop.Active         = 1;
  Stop.OnUpPtr        = 0;
  Stop.OnDownPtr      = StopClick;
  Stop.OnClickPtr     = 0;
  Stop.OnPressPtr     = 0;

  Next.OwnerScreen    = &Screen1;
  Next.Order          = 6;
  Next.Left           = 93;
  Next.Top            = 294;
  Next.Width          = 31;
  Next.Height         = 26;
  Next.Picture_Type   = 0;
  Next.Picture_Ratio  = 1;
  Next.Picture_Name   = Next_next;
  Next.Visible        = 1;
  Next.Active         = 1;
  Next.OnUpPtr        = 0;
  Next.OnDownPtr      = NextClick;
  Next.OnClickPtr     = 0;
  Next.OnPressPtr     = 0;

  Volume.OwnerScreen    = &Screen1;
  Volume.Order          = 7;
  Volume.Left           = 124;
  Volume.Top            = 294;
  Volume.Width          = 31;
  Volume.Height         = 26;
  Volume.Picture_Type   = 0;
  Volume.Picture_Ratio  = 1;
  Volume.Picture_Name   = Volume_volume;
  Volume.Visible        = 1;
  Volume.Active         = 1;
  Volume.OnUpPtr        = 0;
  Volume.OnDownPtr      = VolumeClick;
  Volume.OnClickPtr     = 0;
  Volume.OnPressPtr     = 0;

  Play.OwnerScreen    = &Screen1;
  Play.Order          = 8;
  Play.Left           = 31;
  Play.Top            = 294;
  Play.Width          = 31;
  Play.Height         = 26;
  Play.Picture_Type   = 0;
  Play.Picture_Ratio  = 1;
  Play.Picture_Name   = Play_play;
  Play.Visible        = 1;
  Play.Active         = 1;
  Play.OnUpPtr        = 0;
  Play.OnDownPtr      = PlayClick;
  Play.OnClickPtr     = 0;
  Play.OnPressPtr     = 0;

  Volume_Bar.OwnerScreen    = &Screen1;
  Volume_Bar.Order          = 9;
  Volume_Bar.Left           = 155;
  Volume_Bar.Top            = 294;
  Volume_Bar.Width          = 85;
  Volume_Bar.Height         = 26;
  Volume_Bar.Picture_Type   = 0;
  Volume_Bar.Picture_Ratio  = 1;
  Volume_Bar.Picture_Name   = Volume_Bar_volumebar;
  Volume_Bar.Visible        = 1;
  Volume_Bar.Active         = 1;
  Volume_Bar.OnUpPtr        = 0;
  Volume_Bar.OnDownPtr      = 0;
  Volume_Bar.OnClickPtr     = 0;
  Volume_Bar.OnPressPtr     = VolumeBarChange;

  VolumeBar.OwnerScreen     = &Screen1;
  VolumeBar.Order           = 10;
  VolumeBar.Left            = 162;
  VolumeBar.Top             = 302;
  VolumeBar.Width           = 70;
  VolumeBar.Height          = 9;
  VolumeBar.Pen_Width       = 0;
  VolumeBar.Pen_Color       = 0x0000;
  VolumeBar.Visible         = 1;
  VolumeBar.Active          = 1;
  VolumeBar.Transparent     = 1;
  VolumeBar.Gradient        = 0;
  VolumeBar.Gradient_Orientation    = 0;
  VolumeBar.Gradient_Start_Color    = 0xFFFF;
  VolumeBar.Gradient_End_Color      = 0xC618;
  VolumeBar.Color           = 0xF800;
  VolumeBar.PressColEnabled = 0;
  VolumeBar.Press_Color     = 0x8410;
  VolumeBar.OnUpPtr        = 0;
  VolumeBar.OnDownPtr      = 0;
  VolumeBar.OnClickPtr     = 0;
  VolumeBar.OnPressPtr     = VolumeBarChange;

  Label1.OwnerScreen    = &Screen1;
  Label1.Order          = 11;
  Label1.Left           = 101;
  Label1.Top            = 5;
  Label1.Width          = 39;
  Label1.Height         = 18;
  Label1.Visible        = 1;
  Label1.Active         = 1;
  Label1.Caption        = Label1_Caption;
  Label1.FontName       = Tahoma16x19;
  Label1.Font_Pos_Ver   = 3;
  Label1.Font_Color     = 0x0000;
  Label1.OnUpPtr        = 0;
  Label1.OnDownPtr      = 0;
  Label1.OnClickPtr     = 0;
  Label1.OnPressPtr     = 0;

  Label2.OwnerScreen    = &Screen1;
  Label2.Order          = 12;
  Label2.Left           = 31;
  Label2.Top            = 100;
  Label2.Width          = 176;
  Label2.Height         = 18;
  Label2.Visible        = 1;
  Label2.Active         = 1;
  Label2.Caption        = Label2_Caption;
  Label2.FontName       = Tahoma16x19;
  Label2.Font_Pos_Ver   = 98;
  Label2.Font_Color     = 0xFFFF;
  Label2.OnUpPtr        = 0;
  Label2.OnDownPtr      = 0;
  Label2.OnClickPtr     = 0;
  Label2.OnPressPtr     = 0;

  Label3.OwnerScreen    = &Screen1;
  Label3.Order          = 13;
  Label3.Left           = 55;
  Label3.Top            = 124;
  Label3.Width          = 125;
  Label3.Height         = 18;
  Label3.Visible        = 1;
  Label3.Active         = 1;
  Label3.Caption        = Label3_Caption;
  Label3.FontName       = Tahoma16x19;
  Label3.Font_Pos_Ver   = 122;
  Label3.Font_Color     = 0xFFFF;
  Label3.OnUpPtr        = 0;
  Label3.OnDownPtr      = 0;
  Label3.OnClickPtr     = 0;
  Label3.OnPressPtr     = 0;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetLabel(index)               CurrentScreen->Labels[index]
#define GetImage(index)               CurrentScreen->Images[index]
#define GetBox(index)                 CurrentScreen->Boxes[index]


void DrawLabel(TLabel *Alabel) {
  if (Alabel->Visible == 1) {
    TFT_Set_Font(Alabel->FontName, Alabel->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text_Return_Pos(Alabel->Caption, Alabel->Left, Alabel->Top);
    TFT_Write_Text(Alabel->Caption, (Alabel->Left + ((Alabel->Width - caption_length) / 2)),
                  (Alabel->Top + ((Alabel->Height - caption_height) / 2)));
  }
}

static void DrawLabels() {
  int i;
  TLabel *local_label;

  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    DrawLabel(local_label);
  }
}


void DrawImage(TImage *Aimage) {
  if (Aimage->Visible == 1) {
    if (Aimage->Picture_Type == 0) {
      TFT_Image(Aimage->Left, Aimage->Top, Aimage->Picture_Name, Aimage->Picture_Ratio);
    }
    if (Aimage->Picture_Type == 1) {
      TFT_Image_Jpeg(Aimage->Left, Aimage->Top, Aimage->Picture_Name);
    }
  }
}

static void DrawImages() {
  int i;
  TImage *local_image;

  for ( i = 0 ; i < CurrentScreen->ImagesCount ; i++ ) {
    local_image = GetImage(i);
    DrawImage(local_image);
  }
}


void DrawBox(TBox *ABox) {
  if (ABox->Visible == 1) {
    TFT_Set_Brush(ABox->Transparent, ABox->Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_Start_Color, ABox->Gradient_End_Color);
    TFT_Set_Pen(ABox->Pen_Color, ABox->Pen_Width);
    TFT_Rectangle(ABox->Left, ABox->Top, ABox->Left + ABox->Width - 1, ABox->Top + ABox->Height - 1);
  }
}

static void DrawBoxes() {
  int i;
  TBox *local_box;

  for ( i = 0 ; i < CurrentScreen->BoxesCount ; i++ ) {
    local_box = GetBox(i);
    DrawBox(local_box);
  }
}

void DrawScreen(TScreen *aScreen) {
  unsigned short order;
  unsigned short label_idx;
  TLabel *local_label;
  unsigned short image_idx;
  TImage *local_image;
  unsigned short box_idx;
  TBox *local_box;
  order = 0;
  label_idx = 0;
  image_idx = 0;
  box_idx = 0;

  if ((display_width != aScreen->Width) || (display_height != aScreen->Height)) {
    TFT_BLED_Direction = 0;
    TFT_BLED           = 0;
    TFT_Init(aScreen->Width, aScreen->Height);
    TP_TFT_Init(aScreen->Width, aScreen->Height, 13, 12);                                  // Initialize touch panel
    TP_TFT_Set_ADC_Threshold(900);                              // Set touch panel ADC threshold
    TFT_Fill_Screen(aScreen->Color);
    display_width = aScreen->Width;
    display_height = aScreen->Height;
    TFT_BLED           = 1;
  }
  else
    TFT_Fill_Screen(aScreen->Color);

  CurrentScreen = aScreen;


  while (order < CurrentScreen->ObjectsCount) {
    if (label_idx < CurrentScreen->LabelsCount) {
      local_label = GetLabel(label_idx);
      if (order == local_label->Order) {
        label_idx++;
        order++;
        DrawLabel(local_label);
      }
    }

    if (box_idx < CurrentScreen->BoxesCount) {
      local_box = GetBox(box_idx);
      if (order == local_box->Order) {
        box_idx++;
        order++;
        DrawBox(local_box);
      }
    }

    if (image_idx  < CurrentScreen->ImagesCount) {
      local_image = GetImage(image_idx);
      if (order == local_image->Order) {
        image_idx++;
        order++;
        DrawImage(local_image);
      }
    }

  }
}

static void Process_TP_Press(unsigned int X, unsigned int Y) {
  int i;
  TLabel *local_label;
  TLabel *exec_label;
  short label_order;
  TImage *local_image;
  TImage *exec_image;
  short image_order;
  TBox *local_box;
  TBox *exec_box;
  short box_order;

  label_order         = -1;
  image_order         = -1;
  box_order           = -1;

  // Labels
  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  // Images
  for ( i = 0 ; i < CurrentScreen->ImagesCount ; i++ ) {
    local_image = GetImage(i);
    if (local_image->Active == 1) {
      if (IsInsideObject(X, Y, local_image->Left, local_image->Top,
                         local_image->Width, local_image->Height) == 1) {
        image_order = local_image->Order;
        exec_image = local_image;
      }
    }
  }

  // Boxes
  for ( i = 0 ; i < CurrentScreen->BoxesCount ; i++ ) {
    local_box = GetBox(i);
    if (local_box->Active == 1) {
      if (IsInsideObject(X, Y, local_box->Left, local_box->Top,
                         local_box->Width, local_box->Height) == 1) {
        box_order = local_box->Order;
        exec_box = local_box;
      }
    }
  }

  i = -1;
  if (label_order > i)
    i = label_order;
  if (image_order > i)
    i = image_order;
  if (box_order > i)
    i = box_order;

  if (i != -1) {
    if (i == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnPressPtr != 0) {
          exec_label->OnPressPtr();
          return;
        }
      }
    }

    if (i == image_order) {
      if (exec_image->Active == 1) {
        if (exec_image->OnPressPtr != 0) {
          exec_image->OnPressPtr();
          return;
        }
      }
    }

    if (i == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->OnPressPtr != 0) {
          exec_box->OnPressPtr();
          return;
        }
      }
    }

  }
}

static void Process_TP_Up(unsigned int X, unsigned int Y) {
  int i;
  TLabel *local_label;
  TLabel *exec_label;
  short label_order;
  TImage *local_image;
  TImage *exec_image;
  short image_order;
  TBox *local_box;
  TBox *exec_box;
  short box_order;

  label_order         = -1;
  image_order         = -1;
  box_order           = -1;

  // Labels
  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  // Images
  for ( i = 0 ; i < CurrentScreen->ImagesCount ; i++ ) {
    local_image = GetImage(i);
    if (local_image->Active == 1) {
      if (IsInsideObject(X, Y, local_image->Left, local_image->Top,
                         local_image->Width, local_image->Height) == 1) {
        image_order = local_image->Order;
        exec_image = local_image;
      }
    }
  }

  // Boxes
  for ( i = 0 ; i < CurrentScreen->BoxesCount ; i++ ) {
    local_box = GetBox(i);
    if (local_box->Active == 1) {
      if (IsInsideObject(X, Y, local_box->Left, local_box->Top,
                         local_box->Width, local_box->Height) == 1) {
        box_order = local_box->Order;
        exec_box = local_box;
      }
    }
  }

  i = -1;
  if (label_order > i)
    i = label_order;
  if (image_order > i)
    i = image_order;
  if (box_order > i)
    i = box_order;

  if (i != -1) {
  // Labels
    if (i == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnUpPtr != 0)
          exec_label->OnUpPtr();
        if (PressedObject == (void *)exec_label)
          if (exec_label->OnClickPtr != 0)
            exec_label->OnClickPtr();
        PressedObject = 0;
        return;
      }
    }

  // Images
    if (i == image_order) {
      if (exec_image->Active == 1) {
        if (exec_image->OnUpPtr != 0)
          exec_image->OnUpPtr();
        if (PressedObject == (void *)exec_image)
          if (exec_image->OnClickPtr != 0)
            exec_image->OnClickPtr();
        PressedObject = 0;
        return;
      }
    }

  // Boxes
    if (i == box_order) {
      if (exec_box->Active == 1) {
        if ((exec_box->PressColEnabled == 1) && (PressedObject == (void *)exec_box)) {
          TFT_Set_Brush(exec_box->Transparent, exec_box->Color, exec_box->Gradient, exec_box->Gradient_Orientation, exec_box->Gradient_Start_Color, exec_box->Gradient_End_Color);
          TFT_Set_Pen(exec_box->Pen_Color, exec_box->Pen_Width);
          TFT_Rectangle(exec_box->Left, exec_box->Top, exec_box->Left + exec_box->Width - 1, exec_box->Top + exec_box->Height - 1);
        }
        if (exec_box->OnUpPtr != 0)
          exec_box->OnUpPtr();
        if (PressedObject == (void *)exec_box)
          if (exec_box->OnClickPtr != 0)
            exec_box->OnClickPtr();
        PressedObject = 0;
        return;
      }
    }

  }
  PressedObject = 0;
}

static void Process_TP_Down(unsigned int X, unsigned int Y) {
  int i;
  TLabel *local_label;
  TLabel *exec_label;
  short label_order;
  TImage *local_image;
  TImage *exec_image;
  short image_order;
  TBox *local_box;
  TBox *exec_box;
  short box_order;

  label_order         = -1;
  image_order         = -1;
  box_order           = -1;

  // Labels
  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  // Images
  for ( i = 0 ; i < CurrentScreen->ImagesCount ; i++ ) {
    local_image = GetImage(i);
    if (local_image->Active == 1) {
      if (IsInsideObject(X, Y, local_image->Left, local_image->Top,
                         local_image->Width, local_image->Height) == 1) {
        image_order = local_image->Order;
        exec_image = local_image;
      }
    }
  }

  // Boxes
  for ( i = 0 ; i < CurrentScreen->BoxesCount ; i++ ) {
    local_box = GetBox(i);
    if (local_box->Active == 1) {
      if (IsInsideObject(X, Y, local_box->Left, local_box->Top,
                         local_box->Width, local_box->Height) == 1) {
        box_order = local_box->Order;
        exec_box = local_box;
      }
    }
  }

  i = -1;
  if (label_order > i)
    i = label_order;
  if (image_order > i)
    i = image_order;
  if (box_order > i)
    i = box_order;
  if (i != -1) {
    if (i == label_order) {
      if (exec_label->Active == 1) {
        PressedObject = (void *)exec_label;
        if (exec_label->OnDownPtr != 0) {
          exec_label->OnDownPtr();
          return;
        }
      }
    }

    if (i == image_order) {
      if (exec_image->Active == 1) {
        PressedObject = (void *)exec_image;
        if (exec_image->OnDownPtr != 0) {
          exec_image->OnDownPtr();
          return;
        }
      }
    }

    if (i == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->PressColEnabled == 1) {
          TFT_Set_Brush(exec_box->Transparent, exec_box->Press_Color, exec_box->Gradient, exec_box->Gradient_Orientation, exec_box->Gradient_End_Color, exec_box->Gradient_Start_Color);
          TFT_Set_Pen(exec_box->Pen_Color, exec_box->Pen_Width);
          TFT_Rectangle(exec_box->Left, exec_box->Top, exec_box->Left + exec_box->Width - 1, exec_box->Top + exec_box->Height - 1);
        }
        PressedObject = (void *)exec_box;
        if (exec_box->OnDownPtr != 0) {
          exec_box->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (TP_TFT_Press_Detect()) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
    if (TP_TFT_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Start_TP() {
  InitializeTouchPanel();

  // You can get calibration constants using touch panel calibration example
  TP_TFT_Set_Calibration_Consts(149, 776, 68, 765);    // Set calibration constants

  InitializeObjects();
  display_width = Screen1.Width;
  display_height = Screen1.Height;
  DrawScreen(&Screen1);
}
