#include "mePlayer_objects.h"

//--------------------- User code ---------------------//
#include <built_in.h>
#include "mEPlayer_messages.h"
#include "mEPlayer_playlist_manager.h"
#include "mEPlayer_mp3decoder.h"

const unsigned short MAX_VOLUMEBAR_WIDTH   = 70;

const unsigned short PLAYER_PLAYLIST_NOT_SELECTED = 0;
const unsigned short PLAYER_PLAYLIST_LOADED_OK    = 1;
const unsigned short PLAYER_PLAYLIST_NOT_LOADED   = 2;
const unsigned short PLAYER_FILE_NOT_LOADED       = 3;
const unsigned short PLAYER_FILE_LOADED_OK        = 4;
const unsigned short PLAYER_FILE_NOT_FOUND        = 5;
const unsigned short PLAYER_FAT_INIT_FAILED       = 6;
const unsigned short PLAYER_STOPPED               = 7;
const unsigned short PLAYER_PLAY                  = 8;
const unsigned short PLAYER_PAUSED                = 9;

const unsigned short H_ALIGN_CENTER               = 0;
const unsigned short H_ALIGN_LEFT                 = 0;
const unsigned short H_ALIGN_RIGHT                = 0;

extern TPlaylist    *currentPlaylist;
extern TPlaylist    myPlaylist;
extern unsigned int Xcoord, Ycoord;

static void ShowPlayButton();
static void ShowPauseButton();

void PreviousClick();
void PlayClick();
void StopClick();

void DisplayTimeElapsed(){
  unsigned short minutes, seconds;
  unsigned char txt[5] = "0000";
  unsigned short i;

  if ((currentPlaylist > 0) && (currentPlaylist->PlaylistStatus == PLAYER_PLAYLIST_LOADED_OK)){
    minutes = currentPlaylist->TimeElapsed / 60;
    seconds = currentPlaylist->TimeElapsed % 60;
    ShortToStr(minutes, txt);
    Label1_Caption[0] = txt[2];
    Label1_Caption[1] = txt[3];
    
    ShortToStr(seconds, txt);
    Label1_Caption[2] = ':';
    Label1_Caption[3] = txt[2];
    Label1_Caption[4] = txt[3];
    Label1_Caption[5] = 0;
    for (i = 0; i < 5; i++){
      if (Label1_Caption[i] == ' ')
        Label1_Caption[i] = '0';
    }
    DrawImage(&Header);
    DrawLabel(&Label1);
  }
}

static void AlignLabelsHorizontaly(unsigned short AlignType){
  int newLeftPos;
  
  switch (AlignType){
    case H_ALIGN_CENTER:
      newLeftPos = (240 - 8*strlen(Label2_Caption)) >> 1;
      if (newLeftPos >= 0)
        Label2.Left = newLeftPos;
      else
        Label2.Left = 5;
        
      newLeftPos = (240 - 8*strlen(Label3_Caption)) >> 1;
      if (newLeftPos >= 0)
        Label3.Left = newLeftPos;
      else
        Label3.Left = 5;
      break;
    case H_ALIGN_LEFT:
      break;
    case H_ALIGN_RIGHT:
      break;
  }
}

static void DisplayMessage(const unsigned char * Msg){
  strcpy(Label2_Caption, Msg);
  Label3_caption[0] = 0;
  AlignLabelsHorizontaly(H_ALIGN_CENTER);
  DrawBox(&Box1);
  DrawLabel(&Label2);
  DrawLabel(&Label3);
}

static void DisplayCurrentSongName(){
  strcpy(Label2_Caption, &currentPlaylist->NowPlaying->ArtistName);
  strcpy(Label3_Caption, &currentPlaylist->NowPlaying->SongName);
  AlignLabelsHorizontaly(H_ALIGN_CENTER);
  DrawBox(&Box1);
  DrawLabel(&Label2);
  DrawLabel(&Label3);
}

static void RefreshVolumeBar(){
  unsigned int Volume;
  Set_Volume(currentPlaylist->VolumeLeft, currentPlaylist->VolumeRight);
  VolumeBar.Width = (unsigned int) (0.2745 * (float)currentPlaylist->VolumeLeft);
  DrawImage(&Volume_Bar);
  DrawBox(&VolumeBar);
}
//----------------- End of User code ------------------//

// Event Handlers
void HeaderClick() {
  unsigned short LoadStatus;
  setActivePlaylist(&myPlaylist);
  LoadStatus = loadPlaylist();
  switch (LoadStatus){
    case PLAYER_PLAYLIST_LOADED_OK:
      DisplayTimeElapsed();
      DisplayCurrentSongName();
      RefreshVolumeBar();
      break;
    case PLAYER_PLAYLIST_NOT_LOADED:
      break;
    case PLAYER_PLAYLIST_NOT_SELECTED:
      break;
  }
}

void PreviousClick() {
  unsigned short temp;
  
  if ((currentPlaylist > 0) && (currentPlaylist->PlaylistStatus == PLAYER_PLAYLIST_LOADED_OK)){
    if ((currentPlaylist->FilesCount > 0) && (currentPlaylist->CurrentFileIndex > 0)){
      temp = currentPlaylist->PlayStatus;
      StopClick();
      currentPlaylist->CurrentFileIndex--;
      currentPlaylist->NowPlaying =  &currentPlaylist->Files[currentPlaylist->CurrentFileIndex];
      if (temp == PLAYER_PLAY)
        PlayClick();
      DisplayCurrentSongName();
    }
  }
}


void StopClick() {
  if ((currentPlaylist > 0) && (currentPlaylist->PlaylistStatus == PLAYER_PLAYLIST_LOADED_OK)){
    stopPlaying();
    ShowPlayButton();
    DisplayTimeElapsed();
  }
}

void NextClick() {
  unsigned short temp;
  if ((currentPlaylist > 0) && (currentPlaylist->PlaylistStatus == PLAYER_PLAYLIST_LOADED_OK)){
    if ((currentPlaylist->FilesCount > 0) && (currentPlaylist->CurrentFileIndex < currentPlaylist->FilesCount -1)) {
      temp = currentPlaylist->PlayStatus;
      StopClick();
      currentPlaylist->CurrentFileIndex++;
      currentPlaylist->NowPlaying =  &currentPlaylist->Files[currentPlaylist->CurrentFileIndex];
      if (temp == PLAYER_PLAY)
        PlayClick();
      DisplayCurrentSongName();
    }
  }
}

void VolumeBarChange() {
  int newValue;
  if ((currentPlaylist > 0) && (currentPlaylist->PlaylistStatus == PLAYER_PLAYLIST_LOADED_OK)){
    if (currentPlaylist->Muted == 0){
      newValue = Xcoord - VolumeBar.Left;
      if ((newValue >= 0) && (newValue <= MAX_VOLUMEBAR_WIDTH)){
        currentPlaylist->VolumeLeft = (unsigned short)(((float)newValue / (float)MAX_VOLUMEBAR_WIDTH) * 255.0);
        currentPlaylist->VolumeRight = currentPlaylist->VolumeLeft;
        RefreshVolumeBar();
      }
    }
  }
}

void MuteClick() {
  unsigned short temp;
  if ((currentPlaylist > 0) && (currentPlaylist->PlaylistStatus == PLAYER_PLAYLIST_LOADED_OK)){
    Mute.Visible = 0;
    Mute.Active = 0;
    Volume.Visible = 1;
    Volume.Active = 1;
    currentPlaylist->Muted = 0;
    temp = Mute.Order;
    Mute.Order = Volume.Order;
    Volume.Order = temp;
    DrawImage(&Volume);
  }
}

void VolumeClick() {
  unsigned short temp;
  if ((currentPlaylist > 0) && (currentPlaylist->PlaylistStatus == PLAYER_PLAYLIST_LOADED_OK)){
    Mute.Visible = 1;
    Mute.Active = 1;
    Volume.Visible = 0;
    Volume.Active = 0;
    currentPlaylist->Muted = 1;
    currentPlaylist->VolumeLeft = 0;
    currentPlaylist->VolumeRight = 0;
    temp = Mute.Order;
    Mute.Order = Volume.Order;
    Volume.Order = temp;
    DrawImage(&Mute);
    currentPlaylist->VolumeLeft = 0;
    currentPlaylist->VolumeRight = 0;
    RefreshVolumeBar();
  }
}

static void ShowPlayButton(){
  unsigned short temp;
  Pause.Visible = 0;
  Pause.Active = 0;
  Play.Visible = 1;
  Play.Active = 1;
  temp = Pause.Order;
  Pause.Order = Play.Order;
  Play.Order = temp;
  DrawImage(&Play);
}

static void ShowPauseButton(){
  unsigned short temp;

  Pause.Visible = 1;
  Pause.Active = 1;
  Play.Visible = 0;
  Play.Active = 0;
  temp = Pause.Order;
  Pause.Order = Play.Order;
  Play.Order = temp;
  DrawImage(&Pause);
}

void PauseClick() {
  if ((currentPlaylist > 0) && (currentPlaylist->PlaylistStatus == PLAYER_PLAYLIST_LOADED_OK)){
    pauseFile();
    ShowPlayButton();
  }
}

void PlayClick() {
  unsigned short LoadStatus;
  if ((currentPlaylist > 0) && (currentPlaylist->PlaylistStatus == PLAYER_PLAYLIST_LOADED_OK)){
    if (currentPlaylist->PlayStatus == PLAYER_PAUSED){
      ShowPauseButton();
      playFile();
    }
    else if (currentPlaylist->PlayStatus == PLAYER_STOPPED){
      LoadStatus = loadCurrent();
      switch (LoadStatus){
        case PLAYER_FILE_LOADED_OK:
          ShowPauseButton();
          playFile();
          break;
        case PLAYER_FILE_NOT_FOUND:
          DisplayMessage(MSG_FILE_NOT_FOUND);
          break;
        case PLAYER_FAT_INIT_FAILED:
          DisplayMessage(MSG_NO_CARD);
          break;
        default:
          DisplayMessage(MSG_NO_CARD);
          break;
      }
    }
  }
}
